package gui;

import javax.swing.*;
import java.awt.*;

/**
 * The class which generates the 'X' icon for the tabs. The constructor
 * accepts an icon which is extra to the 'X' icon, so you can have tabs
 * like in JBuilder. This value is null if no extra icon is required.
 */	
class CloseTabIcon implements Icon {
  private int x_pos;
  private int y_pos;
  private int width;
  private int height;
  private Icon fileIcon;
 /**
  * Constructor for CloseTabIcon. Initializes parameters for the 'X' icon
  * for the tabs
  * @param fileIcon
  */
  public CloseTabIcon(Icon fileIcon) {
    this.fileIcon=fileIcon;
    this.width=16;
    this.height=16;
  }
 /**
  * Method for painting the 'X' icon for editor tabs.
  */
  public void paintIcon(Component c, Graphics g, int x, int y) {
    this.x_pos=x;
    this.y_pos=y;
 
    Color col=g.getColor();
    
    g.setColor(Color.black);
    int y_p=y+2;
    g.drawLine(x+1, y_p, x+12, y_p);
    g.drawLine(x+1, y_p+13, x+12, y_p+13);
    g.drawLine(x, y_p+1, x, y_p+12);
    g.drawLine(x+13, y_p+1, x+13, y_p+12);
    g.drawLine(x+3, y_p+3, x+10, y_p+10);
    g.drawLine(x+3, y_p+4, x+9, y_p+10);
    g.drawLine(x+4, y_p+3, x+10, y_p+9);
    g.drawLine(x+10, y_p+3, x+3, y_p+10);
    g.drawLine(x+10, y_p+4, x+4, y_p+10);
    g.drawLine(x+9, y_p+3, x+3, y_p+9);
    g.setColor(col);
    if (this.fileIcon != null) {
      this.fileIcon.paintIcon(c, g, x+this.width, y_p);
    }
  }
 
  public int getIconWidth() {
    return this.width + (this.fileIcon != null? this.fileIcon.getIconWidth() : 0);
  }
 
  public int getIconHeight() {
    return this.height;
  }
 
  public Rectangle getBounds() {
	  return new Rectangle(this.x_pos, this.y_pos, this.width, this.height);
	  
  }
  
}