/**
 * 
 */

package gui;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * This class is the main panel where all the editor tabs are hold.
 * The main user interaction methods such as open, close, save, etc.
 * are provided by guiController object. 
 */
@SuppressWarnings("serial")
public class EditorPane extends JTabbedPaneWithCloseIcons  implements MouseListener {

	public int currentSize;
	public int activeFileIndex;
	public JTabbedPaneWithCloseIcons editor;
	public List <EditorArray> list = new ArrayList<EditorArray>(); 
	public GuiController guiController;
	public PFrame callerFrame;
	
	/**
	 * The constructor for editor panel class. The editor panel gets the 
	 * guiController object and reference of the the main frame of PIDE
	 * which is the caller frame and does the initialization.
	 */
	public EditorPane(GuiController gController, PFrame cFrame) {
		// TODO Auto-generated constructor stub
		guiController = gController;
		callerFrame = cFrame;
		this.initialize();
		}

	/**
	 * This method initializes the appearence and layout properties of the 
	 * editor panel. Also adds a change listener to the panel so that it updates
	 * the active file index. 
	 */
	public void initialize() {
        this.setSize(new Dimension(542, 312));
        this.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);
        
        this.currentSize = 0;
        this.activeFileIndex = -1;
        this.editor = this;
        
        this.addChangeListener(new ChangeListener(){   
			public void stateChanged(ChangeEvent arg0) {
				// TODO Auto-generated method stub
				activeFileIndex = editor.getSelectedIndex();
				System.out.println("activeFileIndex: "+activeFileIndex);
				if(activeFileIndex > -1) 
				{
					callerFrame.getPMenuBar().getSaveMenuItem().setEnabled(true);
					callerFrame.getPMenuBar().getSaveAsMenuItem().setEnabled(true);
					callerFrame.getPToolBar().getSaveButton().setEnabled(true);
					callerFrame.getPToolBar().getSaveAllButton().setEnabled(true);
				}
				else
				{
					callerFrame.getPMenuBar().getSaveMenuItem().setEnabled(false);
					callerFrame.getPMenuBar().getSaveAsMenuItem().setEnabled(false);
					callerFrame.getPToolBar().getSaveButton().setEnabled(false);
					callerFrame.getPToolBar().getSaveAllButton().setEnabled(false);
				}
				//System.out.println("tabcount:"+editor.getTabCount());
				//System.out.println("currently selected: " +	activeFileIndex+ "listLengh: "+list.size());
				//System.out.println(" change flag: "+ list.get(activeFileIndex).changeFlag );
			}		
        });


	}
	/**
	 * This method provides a save dialog for an editor
	 * @param caller frame
	 * @return response (0=NO 1=YES 2=CANCEL)
	 */
	private int saveChangesDialog(JFrame caller) {

		String message = "The file has been modified. Do you want to save changes?";
		int yes = JOptionPane.showConfirmDialog(caller, message);
//		System.out.println("yes: "+yes);
		if( yes == 0) return 0;
		else if(yes == 1)return 1;
		else  return 2;
	}
	/**
	 * The overridden event handler method. Handles the event generated when a mouse
	 * button is clicked. This method is required to handle the closing and disposure of
	 * editor array tabs.
	 */
	public void mouseClicked(MouseEvent e) {
		    
		  //int tabNumber=this.getUI().tabForCoordinate(this, e.getX(), e.getY());
		  int tabNumber = activeFileIndex;  
		  //System.out.println("tabNumber = " + tabNumber);
		    
		    if (tabNumber < 0) 
		    	return;
		    CloseTabIcon temp = (CloseTabIcon) this.getIconAt(tabNumber);
		    
		    
		    temp.getBounds();
		    
		    Rectangle rect=temp.getBounds();
		    
		    if (rect.contains(e.getX(), e.getY())) {
		      //the tab is being closed
		    	//System.out.println("safsdf");
		    	System.out.println("changeFlag : "+this.list.get(tabNumber).changeFlag);
		    	if(this.list.get(tabNumber).changeFlag == 1)
		    	{
		    		int s = saveChangesDialog(callerFrame);
		    		if(s == 0)
		    		{
		    			guiController.saveMenuPressed(callerFrame, this);
		    			list.get(tabNumber).changeFlag = 0;
		    			list.remove(tabNumber);
		    			this.removeTabAt(tabNumber);
		    		}
		    		else if(s == 1)
		    		{
		    			list.remove(tabNumber);
		    			this.removeTabAt(tabNumber);
		    		}
		    	}
		    	else 
		    	{
		    		list.remove(tabNumber);
		    		this.removeTabAt(tabNumber);
		    	}
		     
		    }
		  }		  
}  //  @jve:decl-index=0:visual-constraint="10,10"
