/**
 * 
 */
package peripherals;

import picmicrocontroller.Register;

/**
 * This class is created in order to keep the information of a single
 *  7-Segment Display.
 * @author maho
 * @since 19.01.2007
 */
public class SevenSegmentDisplay {

	//////////////ATTRIBUTES/////////////////////
	
	/**Seven segment id.This id represents the place of the SSD unit in  SevenSegmentDisplayArray */
	private int ssId;
	/**Seven segment data.Data size is 1 byte and each bi represents on/off information of segments*/
	private Register ssDataReg;
	/**Value representing if the SSD unit is activated during simulation.*/
	private boolean isEnabled;
	/**SSD Data*/
	public boolean[] bits;
	
	/////////////CONSTRUCTORS////////////////////////
	/**
	 * Constructor with default values.
	 * Id should be specified.
	 * @param ssId instance-unique id
	 * @author maho
	 * @since 19.01.2007
	 */
	public SevenSegmentDisplay(int ssId) {
		this.ssId = ssId;
		this.bits = new boolean[8];
		for(int i=0;i<8;i++)
		{
			this.bits[i]=false;
		}
		this.isEnabled = false;
		
		this.ssDataReg = new Register();
		this.ssDataReg.write((byte) 0);
	}	
	
	/**
	 * Main constructor of the SevenSegmentDisplay class.
	 * @param ssId		instance-unique id
	 * @param ssData	initial data to be loaded 
	 * @param isEnabled status of seven segment display
	 * @author maho
	 * @since 19.01.2007
	 */
	public SevenSegmentDisplay(int ssId, Register ssData, boolean isEnabled) {
		this.ssId = ssId;
		this.ssDataReg.write(ssData.read());
		this.isEnabled = isEnabled;
	}
	
	/////////////////METHODS/////////////////////////////
	/**
 	* This methods alters the corresponding SSD element in the virtual card.
 	* Setting enable/disable mode is performed by SevenSegmentDisplayArray object.
 	* @author maho
	* @since 19.01.2007
 	*/

	
	public void update()
	{
		if(!this.isEnabled)
		{
			for(int i=0;i<8;i++)
			{
				this.bits[i]=false;
				
			}
		//	this.dummyChar = "  "; 
		}
		else
		{
			for(int i=0;i<8;i++)
			{
				this.bits[i]= (this.ssDataReg.getBit(i)==1)?true:false;
				
			}
		}
	}
	/////////GETTER SETTER METHODS///////////////////////

	/**
	 * @return the isEnabled
	 * @author maho
	 * @since 19.01.2007
	 */
	public boolean isEnabled() {
		return this.isEnabled;
	}

	/**
	 * @param isEnabled the isEnabled to set
	 * @author maho
	 * @since 19.01.2007
	 */
	public void setEnabled(boolean isEnabled) {
		this.isEnabled = isEnabled;
	}

	/**
	 * A SSD doesn't have a register to hold its data. It reads the data from PIC 
	 * ports (PORTD) and gives output only if it is enabled.
	 * @return the ssData
	 * @author maho
	 * @since 19.01.2007
	 */
	public byte readData() {
		if(this.isEnabled)
			return this.ssDataReg.read();
		else return 0;
	}

	/**
	 * A SSD doesn't have a register to hold its data. It reads the data from PIC 
	 * ports (PORTD) and gives output only if it is enabled.
	 * @param ssData the ssData to set
	 * @author maho
	 * @since 19.01.2007
	 */
	public void writeData(byte ssData) {
		if(this.isEnabled)
			this.ssDataReg.write(ssData);
		else this.ssDataReg.write( (byte) 0) ;
	}

	/**
	 * @return the ssId
	 * @author maho
	 * @since 19.01.2007
	 */
	public int getSsId() {
		return this.ssId;
	}

	/**
	 * @param ssId the ssId to set
	 * @author maho
	 * @since 19.01.2007
	 */
	public void setSsId(int ssId) {
		this.ssId = ssId;
	}
	
}
