package picmicrocontroller;
/**
 * This class represents the A/D converter submodule of a 
 * PIC16F877. It provides the simulation method of the A/D
 * converter.
 *
 */
public class ADConverter {

	protected PIC pic;
	private int conversion_cycle;
	private boolean new_conversion;
	private int channel; 
	private double analog_value;
	private int digital_value;
	/**
	 * Main constructor of A/D converter.
	 * @param pic
	 */
	public ADConverter(PIC pic) {
		this.pic = pic;
		conversion_cycle = 0;
		new_conversion = true;
	}
	/**
	 * This method simulates one cycle of the A/D converter.
	 * A/D converter converts the given analog data into the 
	 * digital format and updates necessarry registers in a 
	 * certain amount of cycles. 
	 *
	 */
	public void simulate() {
		if(pic.dataMemory.AD_enabled()) {
			//System.out.println("    enabled");
			
			if(pic.dataMemory.AD_started()) {
				//System.out.println("    started");
				if(new_conversion) {
					//System.out.println("    new conversion");
					new_conversion = false;
					conversion_cycle = 0;
					channel = pic.dataMemory.AD_channel();
					analog_value = pic.analog[channel];
					//assuming Vref is 5 Volts
					digital_value = (int)((analog_value * 1023) / 5000);	//10 bit conversion is performed, 5V = 5000mV corresponds to 1023
					pic.dataMemory.AD_write(digital_value);		//data memory handles right-left justification
				}
				
				if(!new_conversion && conversion_cycle < 12) {
					conversion_cycle++;
				}
				else if(conversion_cycle == 12){	//this conversion is now finished
					pic.dataMemory.AD_stop();
					new_conversion = true;
				}
				
			}
		}
	}
	
	
}
