/**
 * 
 */
package picmicrocontroller;

/**
 * This class represents the data memory of a PIC16F877.
 * Data memory consists of 4 banks, each having 128 registers.
 * @author maho
 *
 */
public class DataMemory {
	
	////////////////ATTRIBUTES/////////////////////
	/***/
	private Register dataMemory[][];
	private Register PCH;
	protected PIC pic;

	
	/////////////CONSTRUCTORS////////////////////////
	/**
	 * Main constructor that initializes the data memory. 
	 * All registers are filled with 0.
	 */
	public DataMemory(PIC pic) {
		
		this.pic = pic;
		this.dataMemory = new Register[4][128];
		this.PCH = new Register();
		this.PCH.write((byte)0);
		
		for(int i=0;i<4;i++)
		{
			for(int j=0;j<128;j++)
			{
				this.dataMemory[i][j] = new Register();
				this.dataMemory[i][j].write((byte)0);/*clearall kullanmaktan daha efficient burda sifirlamak*/
			}
		}
	}

	/////////////////METHODS/////////////////////////////
	/**
	 * Read the content of the register from a given address.
	 * Bank value should be set before read operation.
	 */
	public byte read (int address) 
	{
		return this.dataMemory[this.getBank()][address].read();
	}
	
	/**
	 * Change the content of the register of a given address.
	 * Bank value should be set before write operation.
	 * Checks and updates duplicate values in the data memory.
	 */
	public void write (int address, byte data)
	{
		int oldBank = this.getBank();

		// check whether we are updating the I/O ports
		byte portdata;
		byte trisdata;
		//System.out.println("oldbank=" + getBank());
		if(oldBank==0)
		{
			if((address >=PIC._PORTA) && (address<=PIC._PORTE))
			{
				
				//port
				portdata = this.dataMemory[PIC.BANK0][address].read();
				trisdata = this.dataMemory[PIC.BANK1][address].read();
				
				//System.out.println("olddata=" + data);
				//System.out.println("tris=" + trisdata + " ~trisdata=" + ~trisdata);
				//System.out.println("portdata=" + portdata);
				data = (byte) ((portdata&trisdata) | (data& (~trisdata)));
				
				
								
			}
			
		}else if(oldBank==2) //checking PORTB in bank 2
		{
			if(address==6)
			{
				//port
				portdata = this.dataMemory[PIC.BANK2][PIC._PORTB].read();
				trisdata = this.dataMemory[PIC.BANK3][PIC._TRISB].read();
				data = (byte) ((portdata&trisdata) | (data&~trisdata));
				//dataMemory[PIC.BANK0][PIC._PORTB].write(data);
			}
		}
		
		if((address == PIC.INDIRECT) || (address == PIC.PCL) || (address== PIC.STATUS) || 
				(address == PIC.FSR) || (address == PIC.PCLATH) || (address == PIC.INTCON))
		{
			for(int i=0 ; i<4 ; i++)
			{
				this.dataMemory[i][address].write(data);
				this.pic.simulatorFrame.memoryFrame.getJTable().getModel().setValueAt(data, address-1, i*2+1);
			}
		}
		else if ((address == PIC.TMR0) || (address == PIC._PORTB) )//write double-copy registers
		{
			this.dataMemory[oldBank][address].write(data);
			this.pic.simulatorFrame.memoryFrame.getJTable().getModel().setValueAt(data, address-1, oldBank*2+1);
			
			this.dataMemory[(oldBank+2)%4][address].write(data);
			this.pic.simulatorFrame.memoryFrame.getJTable().getModel().setValueAt(data, address-1, ((oldBank+2)%4)*2+1);


		}
		else
		{
			this.dataMemory[oldBank][address].write(data);
			if(address < 32)
			{
				this.pic.simulatorFrame.memoryFrame.getJTable().getModel().setValueAt(data, address-1, oldBank*2+1);
			}else
			{
				this.pic.simulatorFrame.memoryFrame.getJTable1().getModel().setValueAt(data, address-32, oldBank*2+1);
			}
		}
		
		
		//System.out.println("data = " + data + " mem=" + dataMemory[oldBank][address].read());
	}
	
	
	/**
	 * Clears the Register at the given address
	 */
	public void clear (int address)
	{
		this.dataMemory[this.getBank()][address].clear();
	}
	
	
	/**
	 * Clears all the Registers in the data memory
	 */
	public void reset ()
	{
		for(int i=0; i<3; i++)
		{
			for(int j=0; j<128; j++)
			{
				this.dataMemory[i][j].clear();
			}
		}	
		this.PCH.clear();
		
		
		// ***** POR (power on reset) values of special purpose registers ***** //
		
		//STATUS = '0001 1xxx'
		this.write(PIC.STATUS,(byte) 24);
		
		//OPTION_REG = '1111 1111'
		this.dataMemory[1][PIC.OPTION_REG].write((byte) 255);
		this.dataMemory[3][PIC.OPTION_REG].write((byte) 255);
		this.pic.simulatorFrame.memoryFrame.getJTable().getModel().setValueAt((byte) 255, PIC.OPTION_REG-1, 3);
		this.pic.simulatorFrame.memoryFrame.getJTable().getModel().setValueAt((byte) 255, PIC.OPTION_REG-1, 7);
		
		//TRISA = '--11 1111'
		this.dataMemory[1][PIC._TRISA].write((byte) 63);
		this.pic.simulatorFrame.memoryFrame.getJTable().getModel().setValueAt((byte) 63, PIC._TRISA-1, 3);
		
		//TRISB = TRISC = TRISD = '1111 1111'
		this.dataMemory[1][PIC._TRISB].write((byte) 255);
		this.dataMemory[3][PIC._TRISB].write((byte) 255);
		this.pic.simulatorFrame.memoryFrame.getJTable().getModel().setValueAt((byte) 255, PIC._TRISB-1, 3);
		this.pic.simulatorFrame.memoryFrame.getJTable().getModel().setValueAt((byte) 255, PIC._TRISB-1, 7);
		this.dataMemory[1][PIC._TRISC].write((byte) 255);
		this.pic.simulatorFrame.memoryFrame.getJTable().getModel().setValueAt((byte) 255, PIC._TRISA-1, 3);
		this.dataMemory[1][PIC._TRISD].write((byte) 255);
		this.pic.simulatorFrame.memoryFrame.getJTable().getModel().setValueAt((byte) 255, PIC._TRISA-1, 7);
		
		//TRISE = '0000 -111'
		this.dataMemory[1][PIC._TRISE].write((byte) 7);
		this.pic.simulatorFrame.memoryFrame.getJTable().getModel().setValueAt((byte) 7, PIC._TRISE-1, 3);
		
		//PR2 = '1111 1111'
		this.dataMemory[1][PIC.PR2].write((byte) 255);
		this.pic.simulatorFrame.memoryFrame.getJTable().getModel().setValueAt((byte) 255, PIC.PR2-1, 3);
		
		//TXSTA = '0000 -010'
		this.write(PIC.TXSTA, (byte) 2);
		this.dataMemory[1][PIC.TXSTA].write((byte) 2);
		this.pic.simulatorFrame.memoryFrame.getJTable().getModel().setValueAt((byte) 2, PIC.TXSTA-1, 3);
		
		
		
		//System.out.println("DataMemory.reset()");
	}
	
	/**
	 * Get the current bank number.
	 */
	private int getBank()
	{
		
		int bank = this.dataMemory [0][3].read();
		//System.out.println("bnk" + bank);
		bank = (bank>>5) & 3;
		return bank;
	}
	
	/**
	 * Get content of PORTA register.
	 */
	public Register getPORTA()
	{
		return this.dataMemory[0][PIC._PORTA];
	}
	/**
	 * Get content of PORTB register. 
	 */
	public Register getPORTB()
	{
		return this.dataMemory[0][PIC._PORTB];
	}
	/**
	 * Get content of PORTC register.
	 */
	public Register getPORTC()
	{
		return this.dataMemory[0][PIC._PORTC];
	}
	/**
	 * Get content of PORTD register.
	 */
	public Register getPORTD()
	{
		return this.dataMemory[0][PIC._PORTD];
	}
	/**
	 * Get content of PORTE register.
	 */
	public Register getPORTE()
	{
		return this.dataMemory[0][PIC._PORTE];
	}
	
	
	/**
	 * Get content of TRISA register.
	 */
	public Register getTRISA()
	{
		return this.dataMemory[1][PIC._PORTA];
	}
	/**
	 * Get content of TRISB register.
	 */
	public Register getTRISB()
	{
		return this.dataMemory[1][PIC._PORTB];
	}
	/**
	 * Get content of TRISC register.
	 */
	public Register getTRISC()
	{
		return this.dataMemory[1][PIC._PORTC];
	}
	/**
	 * Get content of TRISD register.
	 */
	public Register getTRISD()
	{
		return this.dataMemory[1][PIC._PORTD];
	}
	/**
	 * Get content of TRISE register.
	 */
	public Register getTRISE()
	{
		return this.dataMemory[1][PIC._PORTE];
	}
	/**
	 * Get content of ADCON0 register.
	 */
	public Register getADCON0()
	{
		return this.dataMemory[0][PIC.ADCON0];
	}
	/**
	 * Get content of PCH register.
	 */
	public byte getPCH() {
		return this.PCH.read();
	}

	/**
	 * Set content of PCH register.
	 */
	public void setPCH(byte data) {
		this.PCH.write(data);
	}
	
	
	//functions to adjust the status bits
	/**
	 * Set the zero flag of the status register.
	 */
	public void setZeroFlag() {
		this.write(3, (byte)(this.read(3) | Byte.parseByte("00000100", 2)));
	}
	/**
	 * Set the zero flag of the status register to 0.
	 */
	public void clearZeroFlag() {
		
		this.write(3, (byte)(this.read(3) & Integer.parseInt("11111011", 2)));
	}
	/**
	 * Set the carry flag of the status register.
	 */
	public void setCarry() {
		this.write(3, (byte)(this.read(3) | Byte.parseByte("00000001", 2)));
	}
	/**
	 * Set the carry flag of the status register to 0.
	 */
	public void clearCarry() {
		this.write(3, (byte)(this.read(3) & Integer.parseInt("11111110", 2)));
	}
	/**
	 * Set the digit flag of the status register.
	 */
	public void setDigitCarry() {
		this.write(3, (byte)(this.read(3) | Byte.parseByte("00000010", 2)));
	}
	/**
	 * Set the digit flag of the status register to 0.
	 */
	public void clearDigitCarry() {
		this.write(3, (byte)(this.read(3) & Integer.parseInt("11111101", 2)));
	}

	/**
	 * Checks if AD converter module is activated.
	 */
	public boolean AD_enabled() {
		boolean result = false;
		int b = this.dataMemory[PIC.BANK0][PIC.ADCON0].read();
		b = b & Integer.parseInt("00000001");
		if(b == 1)
			result = true;
		return result;
	}
	/**
	 * Checks if AD conversion started.
	 * 
	 */
	public boolean AD_started() {
		boolean result = false;
		int b = this.dataMemory[PIC.BANK0][PIC.ADCON0].read();
		b = b & Integer.parseInt("00000100");
		b = b >>> 2;
		if(b == 1)
			result = true;
		return result;
	}
	
	/**
	 * Get the active AD converter channel.
	 * 
	 */
	public int AD_channel() {
		int b = this.dataMemory[PIC.BANK0][PIC.ADCON0].read();
		b = b & Integer.parseInt("00111000");
		b = b >>> 3;
		
		return b;
	}
	/**
	 * Writes the conversion result into ADRESL and ADRESH registers.
	 * @param dval digital value produced after conversion
	 */
	public void AD_write(int dval) {
		int b = this.dataMemory[PIC.BANK1][PIC.ADCON1].read();
		b = b & Integer.parseInt("10000000");
		b = b >>> 7;
		
		if(b == 1) {	//right justified
			int right = dval % 256;
			int left = dval / 256;
			this.dataMemory[PIC.BANK1][PIC.ADRESL].write((byte) right);
			this.dataMemory[PIC.BANK0][PIC.ADRESH].write((byte) left);
		}
		else {		//left justified
			int right = dval % 4;
			right = right << 6;
			int left = dval / 4;
			this.dataMemory[PIC.BANK1][PIC.ADRESL].write((byte) right);
			this.dataMemory[PIC.BANK0][PIC.ADRESH].write((byte) left);
		}
	}
	/**
	 * Stops the AD conversion.
	 *
	 */
	public void AD_stop() {
		this.dataMemory[PIC.BANK0][PIC.ADCON0].write( (byte)(this.dataMemory[PIC.BANK0][PIC.ADCON0].read() - 4));
	}
	
	
	/**
	 * Sets the value of PORTA. This method should be used by peripherals.
	 */
	public void setPORTAbyPeripheral(byte data)
	{
		int trisdata = this.dataMemory[PIC.BANK1][PIC._TRISA].read();
		int portdata = this.dataMemory[PIC.BANK0][PIC._PORTA].read();
		
		data = (byte) ((portdata&(~trisdata)) | (data& trisdata));
		
		this.dataMemory[PIC.BANK0][PIC._PORTA].write(data);
		
	}
	/**
	 * Sets the value of PORTB. This method should be used by peripherals.
	 */
	public void setPORTBbyPeripheral(byte data)
	{
		int trisdata = this.dataMemory[PIC.BANK1][PIC._TRISB].read();
		int portdata = this.dataMemory[PIC.BANK0][PIC._PORTB].read();
		
		data = (byte) ((portdata&(~trisdata)) | (data& trisdata));
		
		
		this.dataMemory[PIC.BANK0][PIC._PORTB].write(data);
		this.dataMemory[PIC.BANK2][PIC._PORTB].write(data);
		
	}
	/**
	 * Sets the value of PORTC. This method should be used by peripherals.
	 */
	public void setPORTCbyPeripheral(byte data)
	{
		int trisdata = this.dataMemory[PIC.BANK1][PIC._TRISC].read();
		int portdata = this.dataMemory[PIC.BANK0][PIC._PORTC].read();
		
		data = (byte) ((portdata&(~trisdata)) | (data& trisdata));
		
		this.dataMemory[PIC.BANK0][PIC._PORTC].write(data);
	}
	/**
	 * Sets the value of PORTD. This method should be used by peripherals.
	 */
	public void setPORTDbyPeripheral(byte data)
	{
		int trisdata = this.dataMemory[PIC.BANK1][PIC._TRISD].read();
		int portdata = this.dataMemory[PIC.BANK0][PIC._PORTD].read();
		
		data = (byte) ((portdata&(~trisdata)) | (data& trisdata));
		
		this.dataMemory[PIC.BANK0][PIC._PORTD].write(data);
	}
	/**
	 * Sets the value of PORTE. This method should be used by peripherals.
	 */
	public void setPORTEbyPeripheral(byte data)
	{
		int trisdata = this.dataMemory[PIC.BANK1][PIC._TRISE].read();
		int portdata = this.dataMemory[PIC.BANK0][PIC._PORTE].read();
		
		data = (byte) ((portdata&(~trisdata)) | (data& trisdata));
		
		this.dataMemory[PIC.BANK0][PIC._PORTE].write(data);
	}
	/**
	 * Gets the value of a register specified by bank number and adress value.
	 * This method should be used when a register is supposed to output a value
	 * to some peripheral.
	 * @param bank
	 * @param address
	 * @return
	 */
	public byte output(int bank, int address)
	{
		return this.dataMemory[bank][address].read();
	}


	
}	
