/**
 * 
 */
package projectManager;

import java.io.File;
import java.io.IOException;

/**
 * This class is the parent class of all file types used bu PIDE.
 * @author maho
 * @since 17.3.2007
 */
public abstract class PIDEFile {
	
	public static final int SYSTEM_SETTINGS = 0;
	public static final int PROJECT = 1;
	public static final int ASM_PLUS_PLUS = 2;
	public static final int ASM = 3;
	public static final int ASM_HEADER = 4;
	public static final int HEX = 5;
	public static final int TEST = 6;
	public static final int DEBUG = 7;
	
	/**Indicates the file's extension*/
	protected int fileType;
	/**Name of the file without extension*/
	protected String fileName;
	/**The full path of the directory where the file resides*/
	protected String filePath;
	/**The buffer that holds a partial content of the file*/
	protected String fileBuffer;
	/**The value that indicates if the file is modified since the last save operation*/
	protected boolean dirty;
	
	
	/**
	 * Default constructor. Required for a default super constructor.
	 * This constructor does not initialize anything.
	 *
	 */
	public PIDEFile()
	{
		
	}
	/**
	 * Main constructor for an abstract PIDE file
	 * @param fileType
	 * @param fileName
	 * @param filePath
	 */
	public PIDEFile(int fileType, String fileName, String filePath) {
		this.fileType = fileType;
		this.fileName = fileName;
		this.filePath = filePath;
		if(!this.filePath.endsWith(File.separator))
		{
			this.filePath += File.separator;
			//System.out.println("pifff "+ this.filePath);
		}
		//this.fileBuffer = new String("#");
		dirty = false;
		
	}
	
	/**
	 * This method saves the content of the fileBuffer
	 * into actual file defined by class attributes.
	 * Also clears the dirty flag.
	 *
	 */
	public abstract void save();
	
	/**
	 * This method loads the actual file into the fileBuffer.
	 * If the specified file is active and dirty, then method 
	 * asks the user if s/he wants to discard changes.
	 *
	 */
	public abstract void load();
	
	/**
	 * This method closes the file. If the file is
	 * dirty, then method asks the user if s/he wants to 
	 * save changes. Returns a value representing if the user
	 * wants to close the file or not. The method handles the saving
	 * operation itself, return value can be used by EditorPane to actually
	 * close the file tab. Valid return values are:
	 * 0 - Do not close file 
	 * 1 - Close file
	 * -1 - Error
	 *  @return a value representing if the user wants to close the file or not. 
	 *
	 */
	public int closeFile()
	{
		int response = 0;
		if(isDirty())
		{
			System.out.println("Do you want to save file before closing? ( (Y)es/(N)o/(C)ancel )");
			try {
				response = System.in.read();
				while( (response  != 'y' || response != 'Y')
					&& (response != 'n' || response != 'N')
					&& (response != 'c' || response != 'C'))
				{
					System.out.println("Invalid answer. Please try one of the appropriate answers. (Yes/No/Cancel)");
					response = System.in.read();
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				return -1;
			}
			if((response  == 'y' || response == 'Y'))
			{
				save();				
			}
			if((response  == 'c' || response == 'C'))
			{
				return 0;			
			}
		}
		return 1;
	}

	/**
	 * Returns if the file is modified
	 * @return booelean value that indicates if there is a change in the file
	 */
	public boolean isDirty() {
		return dirty;
	}

	/**
	 * Method to change file's dirty status
	 * @param dirty 
	 */
	public void setDirty(boolean dirty) {
		this.dirty = dirty;
	}

	/**
	 * @return the fileName
	 */
	public String getFileName() {
		return fileName;
	}

	/**
	 * @param fileName the fileName to set
	 */
	public void setFileName(String fileName) {
		this.fileName = fileName;
	}

	/**
	 * @return the filePath
	 */
	public String getFilePath() {
		return filePath;
	}

	/**
	 * @return the fileType
	 */
	public int getFileType() {
		return fileType;
	}

	/**
	 * @return the fileBuffer
	 */
	public String getFileBuffer() {
		return fileBuffer;
	}
	
	

}
