/*
	First Pass will replace all identifiers declared with EQU with the constant 
	bi tane labeltable bi tane de symboltable gerekiyo

*/


header {
	package compiler;
	import java.io.*;
	import antlr.*;
	import antlr.collections.AST;
	import antlr.debug.misc.ASTFrame;
}

class asmppLexer extends Lexer;

options {
	k = 2;
	charVocabulary = '\0'..'\377';
	testLiterals = false;
}


tokens
{
	// compiler commands:
		BEGIN 	="begin";
		END 	="end";
		EQU 	="equ"; 
		ORG		="org";
		
	// asm++ instructions:
		ADDFF	="addff";
		SUBFF	="subff";	
		SUBWFF	="subwff";
		ADDWFF	="addwff";
		SWAPFF	="swapff";
		IORWFF	="iorwff";
		ANDWFF	="andwff"; 
		XORWFF	="xorwff"; 
		MOVFF	="movff"; 
	// asm++ control structures:
		IF		="if"; 
		ENDIF	="endif";
		ELSE	="else";
		FOR		="for"; 
		WHILE	="while"; 
		DO		="do"; 
		CONTINUE="continue";
		BREAK	="break"; 
	// asm++ declarations	
		FUNCTION ="function";
		VAR		="var"; 
		ARRAY	="array";
		
	// ASM instructions:
		ADDWF	="addwf";
		ANDWF	="andwf";
		CLRF	="clrf";
		CLRW	="clrw";
		COMF	="comf";
		DECF	="decf";
		DECFSZ	="decfsz";	
		INCF	="incf";
		INCFSZ	="incfsz";
		IORWF	="iorwf";
		MOVF	="movf";
		MOVWF	="movwf";
		NOP		="nop";
		RLF		="rlf";
		RRF		="rrf";
		SUBWF	="subwf";
		SWAPF	="swapf";
		XORWF	="xorwf";
		BCF		="bcf";
		BSF		="bsf";
		BTFSC	="btfsc";
		BTFSS	="btfss";
		ADDLW	="addlw";
		ANDLW	="andlw";
		CALL	="call";
		CLRWDT	="clrwdt";
		GOTO	="goto";
		IORLW	="iorlw";
		MOVLW	="movlw";
		RETFIE	="retfie";
		RETLW	="retlw";
		RETURN	="return";
		SLEEP	="sleep";
		SUBLW	="sublw";
		XORLW	="xorlw";

}

{	
	public static void main(String[] args) {
		compile("./deneme");
	}
		
	public static void compile(String fileName) {	
	    // Use a try/catch block for parser exceptions
	    try {
			DataInputStream input = new DataInputStream(new FileInputStream(new File(fileName + ".txt")));
			asmppLexer lexer = new asmppLexer(input); 
			asmppParser parser = new asmppParser(lexer); 
			
			parser.program();
			
			AST tree = parser.getAST();
			//System.out.println(tree.toStringList());
			
/*			
			CommonAST parseTree = (CommonAST)parser.getAST();
			ASTFrame frame = new ASTFrame("The tree", parseTree);
			frame.setSize(350, 1000);
			frame.setLocation(930, 0);
			frame.setVisible(true);
*/			
			// First traversal on tree
			TreeWalker coonWalker = new TreeWalker();				
			String s = coonWalker.program(tree);

			////////////////////////////////////////////////////////
			//WRITE OUTPUT FILE
			System.out.println(s);
			
				File tempFile  = new File(fileName + ".t0");
				FileWriter tempFileWriter = null;
	
				try {
					tempFileWriter = new FileWriter(tempFile);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					System.out.println("System file could not be created! Exiting program.");
					e.printStackTrace();
				}
				try {			
					tempFileWriter.write(s); 
					tempFileWriter.close();
				
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			////////////////////////////////////////////////////////			
			
			
	    }
	    
	    catch(Exception e) {
	      System.err.println("exception: "+e);
	      e.printStackTrace(System.err);   // so we can get stack trace
	    }
	  }
}


	NEWLINE:
		(	'\r' '\n'
		|	'\r'
		|	'\n'
		) {newline();}
	;
	
	WS:
		(	' '
		|	'\t'
		|	'\f'
		//|	NEWLINE {newline();}
		) {$setType(Token.SKIP);}
	;
	
	COMMENT:
		(	("//" | SEMI) (~('\r'|'\n'))* NEWLINE 
		)	{$setType(Token.SKIP);}
	;

	LCURLY:  "{";
	RCURLY:  "}";
	LPAR:	"(";
	RPAR:	")";
	COMMA:	",";
	LBRC:	"[";
	RBRC:	"]";
	SEMI:	";";
	COLON:	":";
	ASSIGN:	"=";
	PLUS:	"+";
	MINUS:	"-";
	AND:	"&&";
	OR:		"||";
	EQ:		"==";
	NEQ:	"!=";	
	LT:		"<";	
	GT:		">";	
	LTE:	"<=";	
	GTE:	">=";	
	
	protected DIGIT:
		(	'0'..'9'
		)
	;
	
	protected DECIMAL:
		(DIGIT)+
	;
	
	protected BINARY:
		("b'" ('0'| '1')+ "'") 
	;
	
	protected HEX:
		("0x" (DIGIT | 'A'..'F')+)
	;
	
	protected IDENT:
	{System.out.println("IDENT");
	
	}
	(
		(LETTER)(LETTER | DIGIT)*)
	;
	
	NUMBER  options { testLiterals=true;}: 
		  BINARY	  
		| HEX
		| DECIMAL
 	  	| IDENT {$setType(IDENT);}
	;
	
	ADDRESS:{System.out.println("ADDRESS");
	
	}
	(
		("*"(DECIMAL | HEX | IDENT)))
	;
	
		
	LITERAL:
		NUMBER | ("'" ('\0'..'\377') "'") 
	;
	
	protected LETTER:
		('a'..'z') | ('A'..'Z')
	;

class asmppParser extends Parser;

options { 
	k = 2;
	buildAST = true;
}
tokens {
	PROGRAM;
	SUBPROGRAMBODY;
	BASICDECL;
	VARDECL;
	CONSTDECL;
	ARRAYDECL;
	STATEMENT;
	ORGSTATEMENT;
	INSTRUCTIONSTATEMENT;
	ASMPPSTATEMENT;
	ADDFFSTATEMENT;
	SUBFFSTATEMENT;
	ADDWFFSTATEMENT;
	SUBWFFSTATEMENT;
	SWAPFFSTATEMENT;
	IORWFFSTATEMENT;
	ANDWFFSTATEMENT;
	XORWFFSTATEMENT;
	MOVFFSTATEMENT;
	ASMSTATEMENT;
	ADDWFSTATEMENT;
	ANDWFSTATEMENT;
	CLRFSTATEMENT;
	CLRWSTATEMENT;
	COMFSTATEMENT;
	DECFSTATEMENT;
	DECFSZSTATEMENT;
	INCFSTATEMENT;
	INCFSZSTATEMENT;
	IORWFSTATEMENT;
	MOVFSTATEMENT;
	MOVWFSTATEMENT;
	NOPSTATEMENT;
	RLFSTATEMENT;
	RRFSTATEMENT;
	SUBWFSTATEMENT;
	SWAPFSTATEMENT;
	XORWFSTATEMENT;
	BSFSTATEMENT;
	BCFSTATEMENT;
	BTFSCSTATEMENT;
	BTFSSSTATEMENT;
	ADDLWSTATEMENT;
	ANDLWSTATEMENT;
	CALLSTATEMENT;
	CLRWDTSTATEMENT;
	GOTOSTATEMENT;
	IORLWSTATEMENT;
	MOVLWSTATEMENT;
	RETFIESTATEMENT;
	RETLWSTATEMENT;
	SLEEPSTATEMENT;
	SUBLWSTATEMENT;
	XORLWSTATEMENT;
	EXPR;
	SUBEXPR;
	IFSTATEMENT;
	WHILESTATEMENT;
	FORSUBSTATEMENT;
	FORSTATEMENT;
	DOWHILESTATEMENT;
	ASSIGNMENTSTATEMENT;
	FUNCTIONCALLSTATEMENT;
	RETURNSTATEMENT;
	LABELSTATEMENT;
	FUNCTIONDECLARATION;
}
	program: {System.out.println("program");}
		//PROGRAM  IDENT ASSIGN 
		//	listStatement
			//configStatement
		((NEWLINE!)*
		 subprogramBody 	
		)
		{
			System.out.println("end program");
			#program = #([PROGRAM,"program"], #program);
		}	
	;
  
  
	subprogramBody:{System.out.println("subprogramBody");}
		(( basicDecl (NEWLINE!)+)*
		 ( funcDeclaration (NEWLINE!)+)*	  
			( statement (NEWLINE!)+)*
		)(END)?
		{
			System.out.println("end program");
			#subprogramBody = #([SUBPROGRAMBODY,"subprogramBody"], #subprogramBody);
		}	
	;

	
	basicDecl:{System.out.println("basicDecl");}
		(
			  varDecl
			| arrayDecl  
			| constDecl
		)
		{
			#basicDecl = #([BASICDECL,"basicDecl"], #basicDecl);
		}
	;
	
	varDecl: {System.out.println("varDecl");}
		(VAR! IDENT (NUMBER) 
		)
		{
			#varDecl = #([VARDECL,"varDecl"], #varDecl);
		}
	;
	
	constDecl: {System.out.println("constDecl");}
		(IDENT EQU! NUMBER 
		)
		{
			#constDecl = #([CONSTDECL,"constDecl"], #constDecl);
		}
	;
	
	
	arrayDecl:{System.out.println("arrayDecl");}
		(ARRAY! IDENT LPAR! NUMBER (COMMA! NUMBER)? RPAR!  
		)
		{
			#arrayDecl = #([ARRAYDECL,"arrayDecl"], #arrayDecl);
		}
	;
	
	statement:		 
		(  instructionStatement 
		 | orgStatement
		 | ifStatement
		 | whileStatement
		 | forStatement
		 | dowhileStatement
		 | assignmentStatement
		 | functioncallStatement
		 | returnStatement
		 | labelStatement
		)
		{
			#statement = #([STATEMENT,"statement"], #statement);
		}
		;
	
	orgStatement:
		( ORG! (NUMBER | IDENT)
		)
		{
			#orgStatement = #([ORGSTATEMENT,"orgStatement"], #orgStatement);
		}
	;
	
	instructionStatement:
		(  asmppStatement
		 | asmStatement
		)
		{
			#instructionStatement = #([INSTRUCTIONSTATEMENT,"instructionStatement"], #instructionStatement);
		}
	; 
	
	asmppStatement:
		(  addffStatement 
		 | subffStatement 
		 | addwffStatement 
		 | subwffStatement
		 | swapffStatement
		 | iorwffStatement
		 | andwffStatement
		 | xorwffStatement
		 | movffStatement
		)
		{
			#asmppStatement = #([ASMPPSTATEMENT,"asmppStatement"], #asmppStatement);
		}
	;
	
	addffStatement:
		( ADDFF! (IDENT | NUMBER) (IDENT | NUMBER)
		)
		{
			#addffStatement = #([ADDFFSTATEMENT,"addffStatement"], #addffStatement);
		}
	;
	
	subffStatement:
		( SUBFF! (IDENT | NUMBER) (IDENT | NUMBER)
		)
		{
			#subffStatement = #([SUBFFSTATEMENT,"subffStatement"], #subffStatement);
		}
	;
	
	addwffStatement:
		( ADDWFF! (IDENT | NUMBER) (IDENT | NUMBER)
		)
		{
			#addwffStatement = #([ADDWFFSTATEMENT,"addwffStatement"], #addwffStatement);
		}
	;
	
	subwffStatement:
		( SUBWFF! (IDENT | NUMBER) (IDENT | NUMBER)
		)
		{
			#subwffStatement = #([SUBWFFSTATEMENT,"subwffStatement"], #subwffStatement);
		}
	;
	
	swapffStatement:
		( SWAPFF! (IDENT | NUMBER) (IDENT | NUMBER)
		)
		{
			#swapffStatement = #([SWAPFFSTATEMENT,"swapffStatement"], #swapffStatement);
		}
	;
	
	iorwffStatement:
		( IORWFF! (IDENT | NUMBER) (IDENT | NUMBER)
		)
		{
			#iorwffStatement = #([IORWFFSTATEMENT,"iorwffStatement"], #iorwffStatement);
		}
	;
	
	andwffStatement:
		( ANDWFF! (IDENT | NUMBER) (IDENT | NUMBER)
		)
		{
			#andwffStatement = #([ANDWFFSTATEMENT,"andwffStatement"], #andwffStatement);
		}
	;
	
	xorwffStatement:
		( XORWFF! (IDENT | NUMBER) (IDENT | NUMBER)
		)
		{
			#xorwffStatement = #([XORWFFSTATEMENT,"xorwffStatement"], #xorwffStatement);
		}
	;
	
	movffStatement:
		( MOVFF! (IDENT | NUMBER) (IDENT | NUMBER)
		)
		{
			#movffStatement = #([MOVFFSTATEMENT,"movffStatement"], #movffStatement);
		}
	;
	
	asmStatement:
		(
		   addwfStatement
		 | andwfStatement
		 | clrfStatement
		 | clrwStatement
		 | comfStatement
		 | decfStatement
		 | decfszStatement
		 | incfStatement
		 | incfszStatement
		 | iorwfStatement
		 | movfStatement
		 | movwfStatement
		 | nopStatement
		 | rlfStatement
		 | rrfStatement
		 | subwfStatement
		 | swapfStatement
		 | xorwfStatement
		 | bcfStatement
		 | bsfStatement
		 | btfscStatement
		 | btfssStatement
		 | addlwStatement
		 | andlwStatement
		 | callStatement
		 | clrwdtStatement
		 | gotoStatement
		 | iorlwStatement
		 | movlwStatement
		 | retfieStatement
		 | retlwStatement
		 //| returnStatement
		 | sleepStatement
		 | sublwStatement
		 | xorlwStatement	
		)
		{
			#asmStatement = #([ASMSTATEMENT,"asmStatement"], #asmStatement);
		} 
	;
	
	addwfStatement:
		( ADDWF! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#addwfStatement = #([ADDWFSTATEMENT,"addwfStatement"], #addwfStatement);
		}
	;

	andwfStatement:
		( ANDWF! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#andwfStatement = #([ANDWFSTATEMENT,"andwfStatement"], #andwfStatement);
		}
	;

	clrfStatement:
		( CLRF! (IDENT | NUMBER)
		)
		{
			#clrfStatement = #([CLRFSTATEMENT,"clrfStatement"], #clrfStatement);
		}
	;
	
	clrwStatement:
		( CLRW! )
		{
			#clrwStatement = #([CLRWSTATEMENT,"clrwStatement"], #clrwStatement);
		}
	;
	
	comfStatement:
		( COMF! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#comfStatement = #([COMFSTATEMENT,"comfStatement"], #comfStatement);
		}
	;
	
	decfStatement:
		( DECF! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#decfStatement = #([DECFSTATEMENT,"decfStatement"], #decfStatement);
		}
	;
	
	decfszStatement:
		( DECFSZ! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#decfszStatement = #([DECFSZSTATEMENT,"decfszStatement"], #decfszStatement);
		}
	;
	
	incfStatement:
		( INCF! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#incfStatement = #([INCFSTATEMENT,"incfStatement"], #incfStatement);
		}
	;
	
	incfszStatement:
		( INCFSZ! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#incfszStatement = #([INCFSZSTATEMENT,"incfszStatement"], #incfszStatement);
		}
	;
	
	iorwfStatement:
		( IORWF! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#iorwfStatement = #([IORWFSTATEMENT,"iorwfStatement"], #iorwfStatement);
		}
	;
	
	movfStatement:
		( MOVF! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#movfStatement = #([MOVFSTATEMENT,"movfStatement"], #movfStatement);
		}
	;
	
	movwfStatement:
		( MOVWF! (NUMBER | IDENT) 
		)
		{
			#movwfStatement = #([MOVWFSTATEMENT,"movwfStatement"], #movwfStatement);
		}
	;
	
	nopStatement:
		( NOP!
		)
		{
			#nopStatement = #([NOPSTATEMENT,"nopStatement"], #nopStatement);
		}
	;
	
	rlfStatement:
		(
			RLF! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#rlfStatement = #([RLFSTATEMENT,"rlfStatement"], #rlfStatement);
		}
	;
	
	rrfStatement:
		( RRF! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#rrfStatement = #([RRFSTATEMENT,"rrfStatement"], #rrfStatement);
		}
	;
	
	subwfStatement:
		( SUBWF! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#subwfStatement = #([SUBWFSTATEMENT,"subwfStatement"], #subwfStatement);
		}
	;
	
	swapfStatement:
		( SWAPF! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#swapfStatement = #([SWAPFSTATEMENT,"swapfStatement"], #swapfStatement);
		}
	;
	
	xorwfStatement:
		( XORWF! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#xorwfStatement = #([XORWFSTATEMENT,"xorwfStatement"], #xorwfStatement);
		}
	;
	
	bcfStatement:
		( BCF! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#bcfStatement = #([BCFSTATEMENT,"bcfStatement"], #bcfStatement);
		}
	;
	
	bsfStatement:
		( BSF! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#bsfStatement = #([BSFSTATEMENT,"bsfStatement"], #bsfStatement);
		}
	;
	
	btfscStatement:
		( BTFSC! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#btfscStatement = #([BTFSCSTATEMENT,"btfscStatement"], #btfscStatement);
		}
	;
	
	btfssStatement:
		( BTFSS! (IDENT | NUMBER) (COMMA! (IDENT | NUMBER))?
		)
		{
			#btfssStatement = #([BTFSSSTATEMENT,"btfssStatement"], #btfssStatement);
		}
	;
	
	addlwStatement:
		( ADDLW! (IDENT | NUMBER)
		)
		{
			#addlwStatement = #([ADDLWSTATEMENT,"addlwStatement"], #addlwStatement);
		}
	;
	
	andlwStatement:
		( ANDLW! (IDENT | NUMBER)
		)
		{
			#andlwStatement = #([ANDLWSTATEMENT,"andlwStatement"], #andlwStatement);
		}
	;
	
	callStatement:
		( CALL! (IDENT | NUMBER)
		)
		{
			#callStatement = #([CALLSTATEMENT,"callStatement"], #callStatement);
		}
	;
	
	clrwdtStatement:
		( CLRWDT! 
		)
		{
			#clrwdtStatement = #([CLRWDTSTATEMENT,"clrwdtStatement"], #clrwdtStatement);
		}
	;
	
	gotoStatement:
		( GOTO! (IDENT | NUMBER) 
		)
		{
			#gotoStatement = #([GOTOSTATEMENT,"gotoStatement"], #gotoStatement);
		}
	;
	
	iorlwStatement:
		( IORLW! (IDENT | NUMBER)
		)
		{
			#iorlwStatement = #([IORLWSTATEMENT,"iorlwStatement"], #iorlwStatement);
		}
	;
	
	movlwStatement:
		( MOVLW! (IDENT | NUMBER)
		)
		{
			#movlwStatement = #([MOVLWSTATEMENT,"movlwStatement"], #movlwStatement);
		}
	;
	
	retfieStatement:
		( RETFIE! 
		)
		{
			#retfieStatement = #([RETFIESTATEMENT,"retfieStatement"], #retfieStatement);
		}
	;
	
	retlwStatement:
		( RETLW! (IDENT | NUMBER)
		)
		{
			#retlwStatement = #([RETLWSTATEMENT,"retlwStatement"], #retlwStatement);
		}
	;
	
	// ASM++ return ile nasil ayiracaz?
	//returnStatement:
	//	RETURN 
	//;
	
	sleepStatement:
		( SLEEP!
		)
		{
			#sleepStatement = #([SLEEPSTATEMENT,"sleepStatement"], #sleepStatement);
		}
	;
	
	sublwStatement:
		( SUBLW! (IDENT | NUMBER)
		)
		{
			#sublwStatement = #([SUBLWSTATEMENT,"sublwStatement"], #sublwStatement);
		}
	;
	
	xorlwStatement:
		( XORLW! (IDENT | NUMBER)
		)
		{
			#xorlwStatement = #([XORLWSTATEMENT,"xorlwStatement"], #xorlwStatement);
		}
	;
	


 	expr:
 		{
 			System.out.println("expr");
 		}
 		(subexpr |
			(
				(	 (LPAR! subexpr RPAR!) | ADDRESS | NUMBER |IDENT	) 
				(AND | OR ) 
				(	(LPAR! subexpr RPAR!)	| ADDRESS | NUMBER|IDENT)
			) 
		
		)
		{
			#expr = #([EXPR,"expr"], #expr);
		}
	;
	

	subexpr:
	{
			System.out.println("subexpr");
	}
		(
			  (((ADDRESS | NUMBER |IDENT) (EQ | NEQ | LT | GT | LTE | GTE )   (ADDRESS | NUMBER|IDENT)) )
		 	| ADDRESS | NUMBER |IDENT
		 )
	// | (LPAR expr RPAR)
		{
			#subexpr = #([SUBEXPR,"subexpr"], #subexpr);
		}
	;
	
	
	ifStatement:
	{
		System.out.println("if");
	}
	
		(IF! LPAR! expr RPAR! 
		 	(NEWLINE!)* (statement (NEWLINE!)+)*
		 (ELSE	(NEWLINE!)* (statement (NEWLINE!)+)*)?
		 ENDIF!
		)
		{
			#ifStatement = #([IFSTATEMENT,"ifStatement"], #ifStatement);
		}
	;
	
		
	
	whileStatement:
	{
		System.out.println("while");
	}
		( WHILE! LPAR! expr RPAR! 
			(NEWLINE!)* LCURLY! (NEWLINE!)* (statement (NEWLINE!)+)*  RCURLY! 
		)
		{
			#whileStatement = #([WHILESTATEMENT,"whileStatement"], #whileStatement);
		}
	;
	
	protected forSubStatement:
   	 	(  assignmentStatement
		 | functioncallStatement   
		 | instructionStatement
		)
		{
			#forSubStatement = #([FORSUBSTATEMENT,"forSubStatement"], #forSubStatement);
		}
	;
 	

 	forStatement:
 		( FOR! LPAR! (forSubStatement)? COLON! expr COLON! (forSubStatement)? RPAR! 
				(NEWLINE!)* LCURLY! (NEWLINE!)* (statement (NEWLINE!)+)*  RCURLY! 
		)
		{
			#forStatement = #([FORSTATEMENT,"forStatement"], #forStatement);
		}
	;
 	
 	dowhileStatement:
 		( DO! 
			(NEWLINE!)* LCURLY! (NEWLINE!)* (statement (NEWLINE!)+)*  RCURLY! 
 		 WHILE! LPAR! expr RPAR!
 		)
 		{
			#dowhileStatement = #([DOWHILESTATEMENT,"dowhileStatement"], #dowhileStatement);
		}
 	;	
 	
 	assignmentStatement:
 		( (ADDRESS | IDENT) ASSIGN! (ADDRESS | IDENT | NUMBER)
 		)
 		{
			#assignmentStatement = #([ASSIGNMENTSTATEMENT,"assignmentStatement"], #assignmentStatement);
		}
 	;
 	
 	functioncallStatement:
 		( IDENT LPAR! ((ADDRESS|IDENT|NUMBER) (COMMA! (ADDRESS|IDENT|NUMBER))* )? RPAR!
 		)
 		{
			#functioncallStatement = #([FUNCTIONCALLSTATEMENT,"functioncallStatement"], #functioncallStatement);
		}
 	;
 	
 	
	returnStatement:
		// assembly RETURN'den ayirmamiz lazim
		( RETURN! ( (ADDRESS | IDENT | NUMBER))?  
		)
		{
			#returnStatement = #([RETURNSTATEMENT,"returnStatement"], #returnStatement);
		}
	;
	
	labelStatement:
		( IDENT COLON!
		)
		{
			#labelStatement = #([LABELSTATEMENT,"labelStatement"], #labelStatement);
		}
	;
	
 	
 	funcDeclaration:
		( FUNCTION! IDENT LPAR! (IDENT (","! IDENT)*)? RPAR! 
		 (NEWLINE!)* LCURLY! (NEWLINE!)* (statement (NEWLINE!)+)*  RCURLY! 
		)
		{
			#funcDeclaration = #([FUNCTIONDECLARATION,"funcDeclaration"], #funcDeclaration);
		}
 	;
 	


{
import java.util.ArrayList;
import java.util.Stack;
}
class TreeWalker extends TreeParser;

options {
 	buildAST = true;	
//	defaultErrorHandler = false;
	k = 1;
}

{
	
	Stack ifStack = new Stack();
	int passNumber=1;
	int addressNumber=0;
	String HEXString = "";
	int hexword=0;
	SymbolTable sTable = new SymbolTable();

	int ifCount = 0;
	

	
	public SymbolTable getSTable() {
		return sTable;
	}
	
	public void setSTable(SymbolTable sT) {
		sTable = sT;
		sTable.printTable();
	}
	
	public void Initialize(int pNum)
	{
		addressNumber=0;
		HEXString = "";
		hexword=0;
		passNumber = pNum;
	}
	
	public int str2int(String str)
	{
			String tempval="";
			int data=0;

			if((str.length() > 1)&& (str.charAt(1)=='x'))
			{	
				//hex
				tempval = str.substring(2);
				data = Integer.parseInt(tempval, 16);
			}else if((str.charAt(0)=='b') | (str.charAt(0)=='B'))
			{ //binary
				tempval = str.substring(2, str.length()-1);
				/*System.out.println("BINARY:"+tempval);*/
				data = Integer.parseInt(tempval, 2);
				
			}else
			{
				if((str.charAt(0)=='d') | (str.charAt(0)=='D'))
				{ //D'10'
					tempval = str.substring(2, str.length()-1);
					data = Integer.parseInt(tempval, 10);
				
				}else //standard decimal
					data = Integer.parseInt(str);
			}
			
			return data;
	}
	public class Symbol {
		private String name;
		private int value;
		// 1:constant, 2:variable, 3:array
		private int type;
		private int startAddress;
		private int arraySize;
		
		Symbol (String nm, int val, int tp, int strAdd, int arrSz) {
			name = nm;
			value = val;
			type = tp;
			startAddress = strAdd;
			arraySize = arrSz;
		}
			
		public void printSymbol() {
			System.out.println("|\t"+name+"\t\t"+value+"\t\t"+type+"\t\t"+startAddress+"\t\t"+arraySize+"\t|");
			System.out.println("+-------------------------------------------------------------------------------+");
		}
	}
	
	public class SymbolTable {
		private ArrayList sTable;
		
		SymbolTable () {
			sTable = new ArrayList();
			
			// Kucuk harf versiyonlari da eklenecek:
			//BANK0
			add("TMR0", "1", 1, -1, 1);
			add("PCL", "2", 1, -1, 1);
			add("STATUS", "3", 1, -1, 1);
				add("RP1", "6", 1, -1, 1);
				add("RP0", "5", 1, -1, 1);
			add("FSR", "4", 1, -1, 1);
			add("PORTA", "5", 1, -1, 1);
			add("PORTB", "6", 1, -1, 1);
			add("PORTC", "7", 1, -1, 1);
			add("PORTD", "8", 1, -1, 1);
			add("PORTE", "9", 1, -1, 1);
			add("PCLATH", "10", 1, -1, 1);
			add("INTCON", "11", 1, -1, 1);
			add("PIR1", "12", 1, -1, 1);
			add("PIR2", "13", 1, -1, 1);
			add("TMR1L", "14", 1, -1, 1);
			add("TMR1H", "15", 1, -1, 1);
			add("T1CON", "16", 1, -1, 1);
			add("TMR2", "17", 1, -1, 1);
			add("T2CON", "18", 1, -1, 1);
			add("SSPBUF", "19", 1, -1, 1);
			add("SSPCON", "20", 1, -1, 1);
			add("CCPR1L", "21", 1, -1, 1);
			add("CCPR1H", "22", 1, -1, 1);
			add("CCP1CON", "23", 1, -1, 1);
			add("RCSTA", "24", 1, -1, 1);
			add("TXREG", "25", 1, -1, 1);
			add("RCREG", "26", 1, -1, 1);
			add("CCPR2L", "27", 1, -1, 1);
			add("CCPR2H", "28", 1, -1, 1);
			add("CCP2CON", "29", 1, -1, 1);
			add("ADRESH", "30", 1, -1, 1);
			add("ADCON0", "31", 1, -1, 1);
			
			//BANK1
			add("OPTION_REG", "1", 1, -1, 1);
			add("TRISA", "5", 1, -1, 1);
			add("TRISB", "6", 1, -1, 1);
			add("TRISC", "7", 1, -1, 1);
			add("TRISD", "8", 1, -1, 1);
			add("TRISE", "9", 1, -1, 1);
			add("PCLATH", "10", 1, -1, 1);
			add("INTCON", "11", 1, -1, 1);
			add("PIE1", "12", 1, -1, 1);
			add("PIE2", "13", 1, -1, 1);
			add("PCON", "14", 1, -1, 1);
			add("SSPCON2", "17", 1, -1, 1);
			add("PR2", "18", 1, -1, 1);
			add("SSPADD", "19", 1, -1, 1);
			add("SSPSTAT", "20", 1, -1, 1);
			add("TXSTA", "24", 1, -1, 1);
			add("SPBRG", "25", 1, -1, 1);
			add("ADRESL", "30", 1, -1, 1);
			add("ADCON1", "31", 1, -1, 1);

			//BANK2
			add("EEDATA", "12", 1, -1, 1);
			add("EEADR", "13", 1, -1, 1);
			add("EEDATH", "14", 1, -1, 1);
			add("EEADRH", "15", 1, -1, 1);

			//BANK3
			add("EECON1", "12", 1, -1, 1);
			add("EECON2", "13", 1, -1, 1);
		}
	
		
		public void add(String name, String value, int type, int startAddress, int arraySize) {
			if(name.charAt(0)=='*')//variable
				name = name.substring(1);
			String tempval="";
			int data=0;

			data = str2int(value);
			
			Symbol sym = new Symbol(name, data, type, startAddress, arraySize);
			sTable.add(sym);
		}
		
		public int getSize() {
			return sTable.size();
		}
		
		public Symbol getSymbol(int i) {
			return (Symbol)sTable.get(i);
		}

		public int getSymbolValue(String s_name) {
			if(s_name.charAt(0)=='*')//variable
				s_name = s_name.substring(1);
			int size = sTable.size();
			Symbol entry = null;
			for (int i = 0; i < size; i++) {
				entry = (Symbol)sTable.get(i);
				if(s_name.compareTo(entry.name) == 0) {
				  return entry.value;
				}
			}
			//Print an error here, there is no such variable 
			System.err.println("No such variable: " + s_name);
			return -1;
			
		} 
		public int getSymbolType(String s_name) {
			int size = sTable.size();
			Symbol entry = null;
			for (int i = 0; i < size; i++) {
				entry = (Symbol)sTable.get(i);
				if(s_name.compareTo(entry.name) == 0) {
				  return entry.type;
				}
			}
			//Print an error here, there is no such variable 
			return -1;
			
		} 
	
		public boolean isDefined (String s_name) {
			int size = sTable.size();
			for (int i = 0; i < size; i++) {
				Symbol entry = (Symbol)sTable.get(i);
				if((s_name.compareTo(entry.name) == 0)) 
				{
				   	return true;
				}
			}
			return false;
		}
				

		public void printTable() {
			System.out.println("+--------------------------------- SYMBOL TABLE  -------------------------------+");
			System.out.println("|\tNAME"+"\t\t"+"VALUE"+"\t\t"+"TYPE"+"\t\t"+"START"+"\t\t"+"SIZE\t|");
			System.out.println("+-------------------------------------------------------------------------------+");
			
			for (int i = 0; i < sTable.size(); i++) {
				getSymbol(i).printSymbol();
			}
		}
	}
}

program returns [String s=""]
{
	s = new String();
	String sub="";
} 

:	#(PROGRAM sub=subprogramBody)
{
	//s += "PROGRAM STARTS:\n";
	s+=sub;
}
;


subprogramBody returns [String s=""]
{
	String dec=null;
	String stat=null;	
	
} 

: 
	#(SUBPROGRAMBODY {/*s += " subprogrambody\n";*/} 
		 (
			(  ((dec=basicDecl)  | (dec=funcDeclaration)) {s += dec + "\n";} 
			)*
			(stat=statement {s += stat;})*
						
		 ) 
	 )
{
		
}
;

basicDecl returns [String s=""]
{
	String dec=null;
}
:
	#(BASICDECL ((dec=arrayDecl) | (dec=varDecl) | (dec=constDecl)))
	{
		//s += " basicdecl ";
		s += dec;
		
	}
;

constDecl returns [String s=""]
{

}
:
	#(CONSTDECL id:IDENT num:NUMBER)
	{
		sTable.add(id.getText(), num.getText(), 1, -1, 1);
		sTable.printTable();
		addressNumber--;
	}
;


arrayDecl returns [String s=""]
{

}
:
	 (#( ARRAYDECL IDENT NUMBER NUMBER)) => #( ARRAYDECL id:IDENT num1:NUMBER num2:NUMBER
	 )	 
	 {	// array F(5,221)
	 	/*
	 	s+=" arraydecl ";
		s += Integer.parseInt(num1.getText());
		s += " ";
		s += Integer.parseInt(num2.getText());
		*/
		
		sTable.add(id.getText(), "-1", 3, str2int(num2.getText()), str2int(num1.getText()));
		sTable.printTable();
		addressNumber--;
	 }
	|
	 (#( ARRAYDECL IDENT NUMBER )) => #( ARRAYDECL id2:IDENT num3:NUMBER
	 )
	 {
	 	/*
		s+=" arraydecl ";
		s += Integer.parseInt(num3.getText());
		*/
		sTable.add(id2.getText(), "-1", 3, -1, str2int(num3.getText()));
		sTable.printTable();
		addressNumber--;
	 }	 	
;


varDecl returns [String s=""]
{

}
:
	 (#( VARDECL IDENT NUMBER)) => #( VARDECL id:IDENT num:NUMBER
	 )	 
	 {
	 	/*
	 	s+=" vardecl ";
		s += Integer.parseInt(num.getText());
		*/
		
		sTable.add(id.getText(), num.getText(), 2, -1, 1);
		sTable.printTable();
		addressNumber--;
	 }
	|
	 (#( VARDECL IDENT )) => #( VARDECL id2:IDENT
	 )
	 {
	 	/*
		s+=" vardecl ";
		*/
		sTable.add(id2.getText(), "-1", 2, -1, 1);
		sTable.printTable();
		addressNumber--;
	 }	 
;

funcDeclaration returns [String s=""]
{
	String s1 = "";
}
:

	#(FUNCTIONDECLARATION {s += " functionDecl ";} 
		(a:IDENT ){s += a.getText() + " ";} 
			(b:IDENT {s += b.getText() + " \n";})* 
				(s1=statement { s += "\t" + s1;})* )  	
	 
;


statement returns [String s=""]
{
	String s1="";
}
:
	
	#(STATEMENT
		(  s1=instructionStatement 
		 | s1=orgStatement	 
		 | s1=ifStatement	 
		 | s1=whileStatement	 
		 | s1=forStatement
		 | s1=dowhileStatement
		 | s1=assignmentStatement 
		 | s1=functioncallStatement
		 | s1=returnStatement
		 | s1=labelStatement
		)
	)
		{
			s += s1 + "\n";
		}	
;

instructionStatement returns [String s=""]
{
	String s1="";
}
:
	#(INSTRUCTIONSTATEMENT (s1=asmppStatement | s1=asmStatement))
	{
		s += s1;
	}
;


asmppStatement returns [String s=""]
{
	String s1="";
}
:
	#(ASMPPSTATEMENT 	(
						  s1=addffStatement 
						| s1=subffStatement
						| s1=addwffStatement						
						| s1=subwffStatement						
						| s1=swapffStatement						
						| s1=iorwffStatement						
						| s1=andwffStatement						
						| s1=xorwffStatement						
						| s1=movffStatement						
						
						)
	 )
	{
		s += s1;
	}
;

addffStatement returns [String s=""]
{
	int v1=0;
	int v2=0;
}
:
	#(ADDFFSTATEMENT {}
		(a:IDENT 
			{
				v1 = sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {v1 = str2int( b.getText());} ) 
			(c:IDENT 
				{
					v2 = sTable.getSymbolValue(c.getText());
				}
			| d:NUMBER {v2 = str2int(d.getText());} ) 
	 )
	 {
	 	s += " movf "  + v2 + ", 0 " + "\n";
	 	s += " addwf " + v1 + ", 1 ";
	 }
	
;

subffStatement returns [String s=""]
{
	int v1=0;
	int v2=0;
}
:
	#(SUBFFSTATEMENT {}
		(a:IDENT 
			{
				v1 = sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {v1 = str2int( b.getText());}  ) 
			(c:IDENT 
				{
					v2 = sTable.getSymbolValue(c.getText());
				}
			| d:NUMBER {v2 = str2int(d.getText());} ) 
	 )
	 {
	 	s += " movf "  + v2 + ", 0 " + "\n";
	 	s += " subwf " + v1 + ", 1 ";
	 }
;


addwffStatement returns [String s=""]
{
	int v1=0;
	int v2=0;
}
:
	#(ADDWFFSTATEMENT {}
		(a:IDENT 
			{
				v1 = sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {v1 = str2int( b.getText());} ) 
			(c:IDENT 
				{
					v2 = sTable.getSymbolValue(c.getText());
				}
			| d:NUMBER {v2 = str2int(d.getText());} ) 
	 )
	{
	 	s += " movf "  + v2 + ", 0 " + "\n";
	 	s += " addwf " + v1 + ", 0 ";
	}
;

subwffStatement returns [String s=""]
{
	int v1=0;
	int v2=0;
}
:
	#(SUBWFFSTATEMENT {}
		(a:IDENT 
			{
				v1 = sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {v1 = str2int( b.getText());} ) 
			(c:IDENT 
				{
					v2 = sTable.getSymbolValue(c.getText());
				}
			| d:NUMBER {v2 = str2int(d.getText());} ) 
	 )
	 {
	 	s += " movf "  + v2 + ", 0 " + "\n";
	 	s += " subwf " + v1 + ", 0 ";
	}	
;

swapffStatement returns [String s=""]
{
	int v1=0;
	int v2=0;	
	int temp=1; // temp yerine ne koyulacak?
				// temp diye bir adres ayirmak lazim
}
:
	#(SWAPFFSTATEMENT {}
		(a:IDENT 
			{
				v1 = sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {v1 = str2int( b.getText());} ) 
			(c:IDENT 
				{
					v2 = sTable.getSymbolValue(c.getText());
				}
			| d:NUMBER {v2 = str2int(d.getText());} ) 
	 )
	{
	 	s += " movf "  + v1 + ", 0 " + "\n";
	 	s += " movwf " +temp + "\n";
	 	s += " movf "  + v2 + ", 0 " + "\n";
	 	s += " movwf " + v1 + "\n";
	 	s += " movf "  +temp+ ", 0" + "\n"; 
	 	s += " movwf " + v2;
	}		
;


iorwffStatement returns [String s=""]
{
	int v1=0;
	int v2=0;	
}
:
	#(IORWFFSTATEMENT {}
		(a:IDENT 
			{
				v1 = sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {v1 = str2int( b.getText());} ) 
			(c:IDENT 
				{
					v2 = sTable.getSymbolValue(c.getText());
				}
			| d:NUMBER {v2 = str2int(d.getText());} ) 
	 )
	{
	 	s += " movf "  + v2 + ", 0 " + "\n";
	 	s += " iorwf " + v1 + ", 1 ";
	}	
;
						
						
						
andwffStatement returns [String s=""]
{
	int v1=0;
	int v2=0;
}
:
	#(ANDWFFSTATEMENT {}
		(a:IDENT 
			{
				v1 = sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {v1 = str2int( b.getText());} ) 
			(c:IDENT 
				{
					v2 = sTable.getSymbolValue(c.getText());
				}
			| d:NUMBER {v2 = str2int(d.getText());} ) 
	 )	
	{
	 	s += " movf "  + v2 + ", 0 " + "\n";
	 	s += " andwf " + v1 + ", 1 ";
	}
;


xorwffStatement returns [String s=""]
{
	int v1=0;
	int v2=0;
}
:
	#(XORWFFSTATEMENT {}
		(a:IDENT 
			{
				v1 = sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {v1 = str2int( b.getText());} ) 
			(c:IDENT 
				{
					v2 = sTable.getSymbolValue(c.getText());
				}
			| d:NUMBER {v2 = str2int(d.getText());} ) 
	 )	
	 {
	 	s += " movf "  + v2 + ", 0 " + "\n";
	 	s += " xorwf " + v1 + ", 1 ";
	}
;


movffStatement returns [String s=""]
{
	int v1=0;
	int v2=0;
}
:
	#(MOVFFSTATEMENT {}
		(a:IDENT 
			{
				v1 = sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {v1 = str2int( b.getText());} ) 
			(c:IDENT 
				{
					v2 = sTable.getSymbolValue(c.getText());
				}
			| d:NUMBER {v2 = str2int(d.getText());} ) 
	 )
	{
	 	s += " movf "  + v1 + ", 0 " + "\n";
	 	s += " movwf " + v2;
	}	
;



asmStatement returns [String s=""]
{	
	String s1 = "";
}
:
	#(ASMSTATEMENT ( 	  s1=addwfStatement
						| s1=andwfStatement
						| s1=clrfStatement
						| s1=clrwStatement
						| s1=comfStatement
						| s1=decfStatement
						| s1=decfszStatement
						| s1=incfStatement
						| s1=incfszStatement
						| s1=iorwfStatement
					 	| s1=movfStatement
					 	| s1=movwfStatement
					 	| s1=nopStatement
					 	| s1=rlfStatement
					 	| s1=rrfStatement
					 	| s1=subwfStatement
					 	| s1=swapfStatement
					 	| s1=xorwfStatement
					 	| s1=bcfStatement
					 	| s1=bsfStatement
					 	| s1=btfscStatement
					 	| s1=btfssStatement
					 	| s1=addlwStatement
					 	| s1=andlwStatement
					 	| s1=callStatement
					 	| s1=clrwdtStatement
					 	| s1=gotoStatement
					 	| s1=iorlwStatement
					 	| s1=movlwStatement
					 	| s1=retfieStatement
					 	| s1=retlwStatement
					 	//| s1=returnStatement
					 	| s1=sleepStatement
					 	| s1=sublwStatement
					 	| s1=xorlwStatement
					)
	 )
	{
		s += s1;
	}	
;

addwfStatement returns [String s=""]
{
}
:
	#(ADDWFSTATEMENT {s += " addwf ";}
		(a:IDENT 
			{	// ident=(constant or variable), doesn't matter => both mean an address
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
			(c:IDENT 
				{ // ident=(constant only, 0 or 1)
					if(sTable.getSymbolType(c.getText()) == 1)
						s += ", " + sTable.getSymbolValue(c.getText());
					else //variable
					{
						System.err.println("Error: Only constants are allowed as destination identifier");
						return "ERROR";
					}						
				}
			| d:NUMBER {s += ", " + d.getText() + " ";} )?
	 )	 
;



andwfStatement returns [String s=""]
{
}
:
	#(ANDWFSTATEMENT {s += " andwf ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
			(c:IDENT 
				{
					// ident=(constant only, 0 or 1)
					if(sTable.getSymbolType(c.getText()) == 1)
						s += ", " + sTable.getSymbolValue(c.getText());
					else //variable
					{
						System.err.println("Error: Only constants are allowed as destination identifier");
						return "ERROR";
					}
				}
			| d:NUMBER {s += ", " + d.getText() + " ";} )?
	 )	 
;


clrfStatement returns [String s=""]
{
}
:
	#(CLRFSTATEMENT {s += " clrf ";}
		(a:IDENT 
			{
				// constant only
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
	 )	 
;

clrwStatement returns [String s=""]
{
}
:
	#(CLRWSTATEMENT {s += " clrw ";}
	 )	 
;



comfStatement returns [String s=""]
{
}
:
	#(COMFSTATEMENT {s += " comf ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
	 )	 
;


decfStatement returns [String s=""]
{
}
:
	#(DECFSTATEMENT {s += " decf ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
	 )	 
;


decfszStatement returns [String s=""]
{
}
:
	#(DECFSZSTATEMENT {s += " decfsz ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
			(c:IDENT 
				{
					// ident=(constant only, 0 or 1)
					if(sTable.getSymbolType(c.getText()) == 1)
						s += ", " + sTable.getSymbolValue(c.getText());
					else //variable
					{
						System.err.println("Error: Only constants are allowed as destination identifier");
						return "ERROR";
					}
				}
			| d:NUMBER {s += ", " + d.getText() + " ";} )?
	 )	 
;


incfStatement returns [String s=""]
{
}
:
	#(INCFSTATEMENT {s += " incf ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
	 )	 
;


incfszStatement returns [String s=""]
{
}
:
	#(INCFSZSTATEMENT {s += " incfsz ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
			(c:IDENT 
				{
					// ident=(constant only, 0 or 1)
					if(sTable.getSymbolType(c.getText()) == 1)
						s += ", " + sTable.getSymbolValue(c.getText());
					else //variable
					{
						System.err.println("Error: Only constants are allowed as destination identifier");
						return "ERROR";
					}
				}
			| d:NUMBER {s += ", " + d.getText() + " ";} )?
	 )	 
;


iorwfStatement returns [String s=""]
{
}
:
	#(IORWFSTATEMENT {s += " iorwf ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
			(c:IDENT 
				{
					// ident=(constant only, 0 or 1)
					if(sTable.getSymbolType(c.getText()) == 1)
						s += ", " + sTable.getSymbolValue(c.getText());
					else //variable
					{
						System.err.println("Error: Only constants are allowed as destination identifier");
						return "ERROR";
					}
				}| d:NUMBER {s += ", " + d.getText() + " ";} )?
	 )	 
;


movfStatement returns [String s=""]
{
}
:
	#(MOVFSTATEMENT {s += " movf ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
			(c:IDENT 
				{
					// ident=(constant only, 0 or 1)
					if(sTable.getSymbolType(c.getText()) == 1)
						s += ", " + sTable.getSymbolValue(c.getText());
					else //variable
					{
						System.err.println("Error: Only constants are allowed as destination identifier");
						return "ERROR";
					}
				}
			| d:NUMBER {s += ", " + d.getText() + " ";} )?
	 )	 
;


movwfStatement returns [String s=""]
{
}
:
	#(MOVWFSTATEMENT {s += " movwf ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
	 )	 
;


nopStatement returns [String s=""]
{
}
:
	#(NOPSTATEMENT {s += " nop ";}
	 )	 
;



rlfStatement returns [String s=""]
{
}
:
	#(RLFSTATEMENT {s += " rlf ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
			(c:IDENT 
				{
					// ident=(constant only, 0 or 1)
					if(sTable.getSymbolType(c.getText()) == 1)
						s += ", " + sTable.getSymbolValue(c.getText());
					else //variable
					{
						System.err.println("Error: Only constants are allowed as destination identifier");
						return "ERROR";
					}
				}
			| d:NUMBER {s += ", " + d.getText() + " ";} )?
	 )	 
;


rrfStatement returns [String s=""]
{
}
:
	#(RRFSTATEMENT {s += " rrf ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
			(c:IDENT 
				{
					// ident=(constant only, 0 or 1)
					if(sTable.getSymbolType(c.getText()) == 1)
						s += ", " + sTable.getSymbolValue(c.getText());
					else //variable
					{
						System.err.println("Error: Only constants are allowed as destination identifier");
						return "ERROR";
					}
				}
			| d:NUMBER {s += ", " + d.getText() + " ";} )?
	 )	 
;


subwfStatement returns [String s=""]
{
}
:
	#(SUBWFSTATEMENT {s += " subwf ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
			(c:IDENT 
				{
					// ident=(constant only, 0 or 1)
					if(sTable.getSymbolType(c.getText()) == 1)
						s += ", " + sTable.getSymbolValue(c.getText());
					else //variable
					{
						System.err.println("Error: Only constants are allowed as destination identifier");
						return "ERROR";
					}
				}
			| d:NUMBER {s += ", " + d.getText() + " ";} )?
	 )	 
;


swapfStatement returns [String s=""]
{
}
:
	#(SWAPFSTATEMENT {s += " swapf ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
			(c:IDENT 
				{
					// ident=(constant only, 0 or 1)
					if(sTable.getSymbolType(c.getText()) == 1)
						s += ", " + sTable.getSymbolValue(c.getText());
					else //variable
					{
						System.err.println("Error: Only constants are allowed as destination identifier");
						return "ERROR";
					}
				}
			| d:NUMBER {s += ", " + d.getText() + " ";} )?
	 )	 
;


xorwfStatement returns [String s=""]
{
}
:
	#(XORWFSTATEMENT {s += " xorwf ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
			(c:IDENT 
				{
					// ident=(constant only, 0 or 1)
					if(sTable.getSymbolType(c.getText()) == 1)
						s += ", " + sTable.getSymbolValue(c.getText());
					else //variable
					{
						System.err.println("Error: Only constants are allowed as destination identifier");
						return "ERROR";
					}
				}
			| d:NUMBER {s += ", " + d.getText() + " ";} )?
	 )	 
;




bcfStatement returns [String s=""]
{
}
:
	#(BCFSTATEMENT {s += " bcf ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
			(c:IDENT 
				{
					// ident=(constant only, 0 or 1)
					if(sTable.getSymbolType(c.getText()) == 1)
						s += ", " + sTable.getSymbolValue(c.getText());
					else //variable
					{
						System.err.println("Error: Only constants are allowed as destination identifier");
						return "ERROR";
					}
				}
			| d:NUMBER {s += ", " + d.getText() + " ";} )?
	 )	 
;


bsfStatement returns [String s=""]
{
}
:
	#(BSFSTATEMENT {s += " bsf ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
			(c:IDENT 
				{
					// ident=(constant only, 0 or 1)
					if(sTable.getSymbolType(c.getText()) == 1)
						s += ", " + sTable.getSymbolValue(c.getText());
					else //variable
					{
						System.err.println("Error: Only constants are allowed as destination identifier");
						return "ERROR";
					}
				}
			| d:NUMBER {s += ", " + d.getText() + " ";} )?
	 )	 
;


btfscStatement returns [String s=""]
{
}
:
	#(BTFSCSTATEMENT {s += " btfsc ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
			(c:IDENT 
				{
					// ident=(constant only, 0 or 1)
					if(sTable.getSymbolType(c.getText()) == 1)
						s += ", " + sTable.getSymbolValue(c.getText());
					else //variable
					{
						System.err.println("Error: Only constants are allowed as destination identifier");
						return "ERROR";
					}
				}
			| d:NUMBER {s += ", " + d.getText() + " ";} )?
	 )	 
;



btfssStatement returns [String s=""]
{
}
:
	#(BTFSSSTATEMENT {s += " btfss ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
			(c:IDENT 
				{
					// ident=(constant only, 0 or 1)
					if(sTable.getSymbolType(c.getText()) == 1)
						s += ", " + sTable.getSymbolValue(c.getText());
					else //variable
					{
						System.err.println("Error: Only constants are allowed as destination identifier");
						return "ERROR";
					}
				}
			| d:NUMBER {s += ", " + d.getText() + " ";} )?
	 )	 
;


addlwStatement returns [String s=""]
{
}
:
	#(ADDLWSTATEMENT {s += " addlw ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
	 )	 
;


andlwStatement returns [String s=""]
{
}
:
	#(ANDLWSTATEMENT {s += " andlw ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
	 )	 
;



callStatement returns [String s=""]
{
}
:
	#(CALLSTATEMENT {s += " call ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
	 )	 
;


clrwdtStatement returns [String s=""]
{
}
:
	#(CLRWDTSTATEMENT {s += " clrwdt ";}
		
	 )	 
;



gotoStatement returns [String s=""]
{
}
:
	#(GOTOSTATEMENT {s += " goto ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
	 )	 
;


iorlwStatement returns [String s=""]
{
}
:
	#(IORLWSTATEMENT {s += " iorlw ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
	 )	 
;
	

movlwStatement returns [String s=""]
{
}
:
	#(MOVLWSTATEMENT {s += " movlw ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
	 )	 
;
	

retfieStatement returns [String s=""]
{
}
:
	#(RETFIESTATEMENT {s += " retfie ";}
	 )	 
;


retlwStatement returns [String s=""]
{
}
:
	#(RETLWSTATEMENT {s += " retlw ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
	 )	 
;
	

sleepStatement returns [String s=""]
{
}
:
	#(SLEEPSTATEMENT {s += " sleep ";}
	 )	 
;
	

sublwStatement returns [String s=""]
{
}
:
	#(SUBLWSTATEMENT {s += " sublw ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
	 )	 
;



xorlwStatement returns [String s=""]
{
}
:
	#(XORLWSTATEMENT {s += " xorlw ";}
		(a:IDENT 
			{
				s += sTable.getSymbolValue(a.getText());
			}
		| b:NUMBER {s += b.getText() + " ";} ) 
	 )	 
;
		 



orgStatement returns [String s=""]
{
}
:
	#(ORGSTATEMENT {/*s += " orgStatement ";*/ s+=" org ";}
		(a:IDENT 
			{
				// only constant values are allowed in an "org" statement
				s += sTable.getSymbolValue(a.getText());
			}
			
		| b:NUMBER {s += b.getText() + " ";} ) 
	 )	
;


ifStatement returns [String s=""]
{
	String s1="";
	String s2="";
	String s3="";
}
:
	#(IFSTATEMENT {ifCount++; ifStack.push(ifCount); }
		(s1=expr {s += s1; s+= "IF" + ifStack.peek() + ":\n";} (s2=statement {s += s2;})* ) 
		(ELSE {s+= "goto ENDIF" + ifStack.peek() + "\n";s+= "ELSE" + ifStack.peek() + ":\n";} (s3=statement {s += s3;})* )?
	 )	
	 {
	 	s+= "ENDIF" + ifStack.peek() + ":\n";
	 	ifStack.pop();	
	 }
;

expr returns [String s=""]
{
	String s1="";
	String s2="";	
	
}
:
	#(EXPR  (  /*(subexpr AND subexpr)	=>	( (s1=subexpr AND s2=subexpr ) 	{s += s1+" AND"+s2+" ";	})
			 | (subexpr OR  subexpr)	=>	( (s1=subexpr OR s2=subexpr ) 	{s += s1+" OR" +s2+" ";	})
			 | (subexpr) 				=>	*/( (s1=subexpr) 					{s += s1;				})
			)
	 )

;


subexpr returns [String s=""]
{
	int opType = 0;
	String v1 = "";
	String v2 = "";
	int op1type = 0; //1=address 2=ident 3=number
	int op2type = 0;
}
:
	#(SUBEXPR (	((ADDRESS | NUMBER | IDENT) 
						(EQ | NEQ | LT | GT | LTE | GTE) (ADDRESS | NUMBER | IDENT))
				 =>	( (a:ADDRESS{op1type =1;} | b:NUMBER{op1type =3;} | c:IDENT{op1type =2;}) 
				 		(EQ {opType=1;} | NEQ {opType=2;}| LT {opType=3;}| GT {opType=4;}| LTE {opType=5;} | GTE{opType=6;})
				 			(d:ADDRESS{op2type =1;} | e:NUMBER{op2type =3;} | f:IDENT{op2type =2;}) )
				 			{
				 				switch(opType)
				 				{
				 				case 1: //EQ
				 					//s += "EQUAL op1:" + op1type +" op2:" + op2type + "\n";
				 					switch(op1type)
				 					{
				 					case 1: //address
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " subwf " + sTable.getSymbolValue(a.getText()) + ", 0\n";	
				 						break;
				 					case 2: //ident
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " sublw " + sTable.getSymbolValue(c.getText()) + "\n";	
				 						break;
				 					case 3: //number
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " sublw " + b.getText() + "\n";	
				 						break;
				 					}
				 					
				 								 					
				 					s += " btfss STATUS, 2\n";
				 					s += " goto ELSE" + ifStack.peek() + "\n";
				 					break;
				 				case 2: //NEQ
				 					//s += "NOT EQUAL op1:" + op1type +" op2:" + op2type + "\n";
				 					switch(op1type)
				 					{
				 					case 1: //address
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " subwf " + sTable.getSymbolValue(a.getText()) + ", 0\n";	
				 						break;
				 					case 2: //ident
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " sublw " + sTable.getSymbolValue(c.getText()) + "\n";	
				 						break;
				 					case 3: //number
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " sublw " + b.getText() + "\n";	
				 						break;
				 					}
				 					
				 								 					
				 					s += " btfsc STATUS, 2\n";
				 					s += " goto ELSE" + ifStack.peek() + "\n";
				 					break;
				 					
				 				case 3: //LT
				 					//s += "LESS THAN op1:" + op1type +" op2:" + op2type + "\n";
				 					switch(op1type)
				 					{
				 					case 1: //address
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " subwf " + sTable.getSymbolValue(a.getText()) + ", 0\n";	
				 						break;
				 					case 2: //ident
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " sublw " + sTable.getSymbolValue(c.getText()) + "\n";	
				 						break;
				 					case 3: //number
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " sublw " + b.getText() + "\n";	
				 						break;
				 					}
				 					
				 								 					
				 					s += " btfsc STATUS, 0\n";
				 					s += " goto ELSE" + ifStack.peek() + "\n";
				 					break;
				 				case 4: //GT
				 					//s += "GREATER THAN op1:" + op1type +" op2:" + op2type + "\n";
				 					switch(op1type)
				 					{
				 					case 1: //address
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " subwf " + sTable.getSymbolValue(a.getText()) + ", 0\n";	
				 						break;
				 					case 2: //ident
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " sublw " + sTable.getSymbolValue(c.getText()) + "\n";	
				 						break;
				 					case 3: //number
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " sublw " + b.getText() + "\n";	
				 						break;
				 					}
				 					
				 								 					
				 					s += " btfss STATUS, 0\n";
				 					s += " goto ELSE" + ifStack.peek() + "\n";
				 					break;
				 				case 5://LTE
				 					//s += "LESS THAN OR EQUAL op1:" + op1type +" op2:" + op2type + "\n";
				 					switch(op1type)
				 					{
				 					case 1: //address
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " subwf " + sTable.getSymbolValue(a.getText()) + ", 0\n";	
				 						break;
				 					case 2: //ident
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " sublw " + sTable.getSymbolValue(c.getText()) + "\n";	
				 						break;
				 					case 3: //number
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " sublw " + b.getText() + "\n";	
				 						break;
				 					}
				 					
				 								 					
				 					s += " btfss STATUS, 2\n";
				 					s += " goto ELSE" + ifStack.peek() + "\n";
				 					s += " btfsc STATUS, 0\n";
				 					s += " goto ELSE" + ifStack.peek() + "\n";
				 					break;
				 				case 6: //GTE
									//s += "GREATER THAN OR EQUAL op1:" + op1type +" op2:" + op2type + "\n";
				 					switch(op1type)
				 					{
				 					case 1: //address
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " subwf " + sTable.getSymbolValue(a.getText()) + ", 0\n";	
				 						break;
				 					case 2: //ident
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " sublw " + sTable.getSymbolValue(c.getText()) + "\n";	
				 						break;
				 					case 3: //number
				 						switch(op2type)
					 					{
					 					case 1: //address
					 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
					 						break;
					 					case 2: //ident
					 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
					 						break;
					 					case 3: //number
					 						s += " movlw "  + e.getText() + "\n";
					 						break;
					 					}
					 					s += " sublw " + b.getText() + "\n";	
				 						break;
				 					}
				 					
				 								 					
				 					s += " btfss STATUS, 2\n";
				 					s += " goto ELSE" + ifStack.peek() + "\n";
				 					s += " btfss STATUS, 0\n";
				 					s += " goto ELSE" + ifStack.peek() + "\n";
				 				default:
				 					System.err.println("Error in sub-expression");
				 					break;				 				
				 				}
				 			}
				| ( (k:ADDRESS{op1type=1;} | l:NUMBER{op1type=3;} | m:IDENT{op1type=2;})
					{
						//s += "SINGLE ELEMENT SUBEXPR op1:" + op1type + "\n";
						switch(op1type)
					 	{
					 		case 1: //address
					 			s += " movf "  + sTable.getSymbolValue(k.getText()) + ", 0\n";
					 			break;
					 		case 2: //ident
					 			s += " movlw "  + sTable.getSymbolValue(m.getText()) + "\n";
					 			break;
					 		case 3: //number
					 			s += " movlw "  + l.getText() + "\n";
					 			break;
					 	}
					 	s += " btfsc STATUS, 2\n";
				 		s += " goto ELSE" + ifStack.peek() + "\n";
					}
				  )
			  )			  
	 )	
;



whileStatement returns [String s=""]
{
	String s1="";
	String s2="";
		
}
:
	#(WHILESTATEMENT {ifCount++; ifStack.push(ifCount); }
		(s1=expr {s += s1; s+= "WHILE" + ifStack.peek() + ":\n";} (s2=statement {s += s2;})* ) 
	 )	
	 {
	 	s+= "goto WHILE" + ifStack.peek() + "\n";
		s+= "ELSE " + ifStack.peek() + ":\n";
		ifStack.pop();	
	 }
;


forStatement returns [String s=""]
{
	String s1="";
	String s2="";
	String s3="";
	String s4="";		
}
:
	#(FORSTATEMENT {ifCount++; ifStack.push(ifCount);}
		(s1=forSubStatement {s += s1+"\n";s+= "FOR" + ifStack.peek() + ":\n";})? 
		(s2=expr 			{s += s2;}) 
		(s3=forSubStatement )? 
		(s4=statement 		{s += s4;})*{s += s3;} 
	 )	
	 {
	 	s+= "\ngoto FOR" + ifStack.peek() + "\n";
		s+= "ELSE " + ifStack.peek() + ":\n";
		ifStack.pop();	
	 }
;



forSubStatement returns [String s=""]
{
	String s1="";
}
:
	#(FORSUBSTATEMENT 	( s1=instructionStatement 
						| s1=functioncallStatement
						| s1=assignmentStatement
						)
	  )	{s += s1;}
;



dowhileStatement returns [String s=""]
{
	String s1="";
	String s2="";	
}
:
	#(DOWHILESTATEMENT {ifCount++; ifStack.push(ifCount);s+= "DOWHILE" + ifStack.peek() + ":\n";}
		((s2=statement {s += s2;})* (s1=expr) {s += s1;}  ) 
	 )	
	 {
	 	s+= "\ngoto DOWHILE" + ifStack.peek() + "\n";
		s+= "ELSE " + ifStack.peek() + ":\n";
		ifStack.pop();	
	 }
;


assignmentStatement returns [String s=""]
{
	int op1type=0;
	int op2type=0;
	String s1="";
	String s2="";	
}
:
	#(ASSIGNMENTSTATEMENT {/*s += " assignmentStatement ";*/}
		((a:ADDRESS{op1type=1;}|b:IDENT{op1type=2;}) (c:ADDRESS{op2type=1;}|d:IDENT{op2type=2;}|e:NUMBER{op2type=3;})) 
	 )	
	 {
	 	s += "op1:" + op1type + " op2:" + op2type + "\n";
	 	
	 	switch(op2type)//RHS
	 	{
	 		case 1: //address
	 			s += "movf " + sTable.getSymbolValue(c.getText()) + "\n";
	 			switch(op1type)//LHS
			 	{
			 		case 1: //address 
		 				System.err.println("Invalid assignment LHS=address");
			 			break;
			 		case 2: //ident
			 			if(sTable.getSymbolType(b.getText()) == 1)
			 				System.err.println("Invalid assignment 1");
			 			else
			 				s += "movwf " + sTable.getSymbolValue(b.getText()) + "\n";			 		
			 			break;
			 	} 
	 			break;
	 		case 2: //ident
	 			s += "movlw " + sTable.getSymbolValue(d.getText()) + "\n";
	 			switch(op1type)
			 	{
			 		case 1:
			 			System.err.println("Invalid assignment LHS=address");
			 			break;
			 		case 2:
			 			if(sTable.getSymbolType(b.getText()) == 1)
			 				System.err.println("Invalid assignment 1");
			 			else
			 				s += "movwf " + sTable.getSymbolValue(b.getText()) + "\n";		
			 			break;
			 	} 
	 			break;
	 		case 3: //number
	 			s += "movlw " + e.getText() + "\n";
	 			switch(op1type)
			 	{
			 		case 1:
						System.err.println("Invalid assignment LHS=address");
			 			break;
			 		case 2:
			 			if(sTable.getSymbolType(b.getText()) == 1)
			 				System.err.println("Invalid assignment 1");
			 			else
			 				s += "movwf " + sTable.getSymbolValue(b.getText()) + "\n";		
			 			break;
			 	} 
	 			break;
	 	} 
	 }
;

functioncallStatement returns [String s=""]
{
		
}
:
	#(FUNCTIONCALLSTATEMENT {s += " functionCallStatement ";}
		(IDENT ((ADDRESS|IDENT|NUMBER) (ADDRESS|IDENT|NUMBER)*)?) 
	 )	
;



returnStatement returns [String s=""]
{
	int optype=0;		
}
:
	#(RETURNSTATEMENT {} 
			(a:ADDRESS{optype=1;}|b:IDENT{optype=2;}|c:NUMBER{optype=3;})?
			
	 )	
	 	{
				switch(optype)
				{
					case 0:
						s += "return";
						break;
					case 1:
						s += "movf " + sTable.getSymbolValue(a.getText()) + ", 0\n";
						s += "retlw";
						break;
					case 2:
						s += "movlw " + sTable.getSymbolValue(b.getText()) + "\n";
						s += "retlw";
						break;
					case 3:
						s += "movlw " + c.getText() + "\n";
						s += "retlw";
						break;
				}
			}
;


labelStatement returns [String s=""]
{
		
}
:
	#(LABELSTATEMENT {} 
		id:IDENT
	 )	
	 {
	 	s += id.getText() + ":" ;
	 }
;



