/**
 * 
 */
package peripherals;

import picmicrocontroller.PIC;
import picmicrocontroller.Register;
import simulator.LEDPanel;
import simulator.SevenSegmentPanel;

/**
 * This class is created in order to keep the information of eight 
 * 7LEDs. The class is inherited from the Peripheral class.
 * @author maho
 * @since 22.04.2007
 */
public class LEDArray extends Peripheral {
	
	//////////////////CONSTANTS////////////////////
	/**Number of LED elements to be simulated*/
	private static int NO_OF_LED = 8;
	//private static int MAX_COUNTER = 20;
	
	////////////////ATTRIBUTES/////////////////////
	/**Array of LEDs. The board has 8 LEDs.*/
	private LED LEDArray[];
	
	private LEDPanel ledPanel;

	//public boolean sticky;
	//private int counter;
	
	
	/////////////CONSTRUCTORS////////////////////////
	/**
	 * Constructor which initializes 8 LED objects with default settings.
	 * @author maho
	 * @since 22.04.2007
	 */
	public LEDArray(PIC pic) {
		
		super(LED_ARRAY, pic, true);
		this.ledPanel = new LEDPanel();
		this.LEDArray = new LED[NO_OF_LED];
		//id 0-->leftmost ... id 8-->rightmost
		for(int i=0;i<NO_OF_LED;i++)
		{
			this.LEDArray[i] = new LED(i);
		}
	}
	/**
	 * Main constructor of LEDArray class.
	 * @param isEnabled status of LEDArray
	 * @author maho
	 * @since 22.04.2007
	 */
	public LEDArray(PIC pic, boolean isEnabled ) {
		super(LED_ARRAY, pic, isEnabled);
		this.ledPanel = new LEDPanel();
		this.LEDArray = new LED[NO_OF_LED];
		
		for(int i=0;i<NO_OF_LED;i++)
		{
			this.LEDArray[i] = new LED(i);
		}
	}
	/////////////////METHODS/////////////////////////////

	@Override
	public void update() {
		
		//System.out.println("LEDArray.update()");
			
		Register output = this.pic.getDataMemory().getPORTD();
		byte trisD = (byte) ~(this.pic.getDataMemory().getTRISD().read());
		//System.out.println("tris: " +trisD);
		output.write((byte) ((byte) output.read() & (byte) trisD));
		//System.out.println("out: " + output.read());
			//byte porteBit =(byte) ((1<<i)&(pic.getDataMemory().getPORTE().read())>>i);
		Register porta = this.pic.getDataMemory().getPORTA();
		//check if leds are enabled. PORTA.2 is the bit to check
		if(porta.getBit(2)== 1)
		{
			for(int i=0;i<NO_OF_LED;i++)
			{
				this.LEDArray[i].setEnabled(true);
				//mask the value of portd with trisd for i/o
				//System.out.println("write: " +output.getBit(i));
				this.LEDArray[i].writeData(output.read());				
				this.LEDArray[i].update();
				this.ledPanel.changeState(i,this.LEDArray[i].bit);
			}
		}
		else
		{
			for(int i=0;i<NO_OF_LED;i++)
			{
				this.LEDArray[i].setEnabled(false);
				//this.LEDArray[i].writeData((byte) 0);
				this.LEDArray[i].update();
				this.ledPanel.changeState(i,this.LEDArray[i].bit);
			}
		}
			
	}
	/**
	 * This method returns the visual represantation of the LED array.
	 * @return KeyPadPanel
	 */
	public LEDPanel getLedPanel() {
		return this.ledPanel;
	}


}
