/**
 * 
 */
package projectManager;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;

/**
 * This class holds information of a PIDE project. 
 * @author maho
 *
 */
public class ProjectFile extends PIDEFile {

	public Project project;
	public String projectDescription;
	public ArrayList<String> fileList;
	/**
	 * This constructor is invoked by system  when a new project
	 * is created. The constructor creates a project file into project folder. 
	 * @param fileType
	 * @param fileName
	 * @param filePath
	 * @param project
	 */
	public ProjectFile(int fileType, String fileName, String filePath, String projectDescription) {
		super(fileType, fileName, filePath);
		this.projectDescription = projectDescription;
		this.project = new Project(this);
		this.fileList = new ArrayList<String>();
		dirty = false;
		this.fileBuffer = new String();
		
		save();
		
	}

	/**
	 * This constructor is invoked by a project.
	 *  The constructor loads data from the existing
	 * project file.
	 * @param project
	 */
	public ProjectFile(Project project)
	{
		
		this.project = project;
		this.fileType = PIDEFile.PROJECT;
		this.fileName = project.getProjectName();
		this.filePath = project.getProjectPath();
		if(!this.filePath.endsWith(File.separator))
		{
			this.filePath += File.separator;
			//System.out.println("pifff "+ this.filePath);
		}
		this.fileList = new ArrayList<String>();
		dirty = false;
		this.fileBuffer = new String("#");
		this.projectDescription = new String();
		load();
		
		
	}
	/* (non-Javadoc)
	 * @see projectManager.PIDEFile#load()
	 */
	@Override
	public void load() {
		// TODO Auto-generated method stub
		String newline = System.getProperty("line.separator");

		File projectFile  = new File(filePath + fileName + ".pde");
		BufferedReader inputStream = null;
		try {
			inputStream = new BufferedReader(new FileReader(projectFile));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			System.out.println("Project file missing! Could not open project " + fileName);
			e.printStackTrace();
		}
		
		String line = new String();
		try {
			//skip comments and other info
			line = inputStream.readLine().substring(1);
			while(!line.contains("#Description")){
				
				
				fileBuffer += (line + newline);
				line = inputStream.readLine();
				 //get very first few comments that will never change after the file is created
			}
		
			
			//after finding the spot, get project decription

			line = line.substring(13);
			line = inputStream.readLine();
			while (!line.contains("#Files"))//until file list is reached
			{
				projectDescription += line;
				line = inputStream.readLine();
				 //read each line into description
			}
			
			//!!check if original project  description is changed!! 
			
			
			line = inputStream.readLine();
			//extract file paths that are included in this project
			while(!line.contains("WORKSPACE_BEGIN")){
				//line = inputStream.readLine();
				//System.out.println("adding file of extension "+ line.substring(1) +" to the project");
				
				//System.out.println("File path is: "+ line.substring(5));
				fileList.add(line.substring(7));
				line = inputStream.readLine();
				//System.out.println();
			}
			//the workspace properties are not implemented yet
			/*
			 System.out.println(fileBuffer);
			if( projectDescription.length() != 0)
				System.out.println(projectDescription);
				*/
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	/* (non-Javadoc)
	 * @see projectManager.PIDEFile#save()
	 */
	@Override
	public void save() {
		// TODO Auto-generated method stub
		//System.out.println(filePath + fileName + ".pde");
		File systemFile  = new File(filePath + fileName + ".pde");
		String newline = System.getProperty("line.separator");
		FileWriter systemFileWriter = null;
		
		try {
			systemFileWriter = new FileWriter(systemFile);
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(0);
		}
		String line = new String();
		try {
			if(fileBuffer.length() != 0)//saving after project is created
			{
				//System.out.println("saving after project is created");
				//System.out.println(fileBuffer);
				systemFileWriter.write(fileBuffer);//initial comments
				systemFileWriter.write("#Modified@ "+ new Date().toString());
				systemFileWriter.write(newline);
				systemFileWriter.write("#Description= ");
				if( projectDescription.length() != 0)
					systemFileWriter.write(projectDescription.replaceAll(">", ""));
				systemFileWriter.write(newline);
				systemFileWriter.write("#Files");
				systemFileWriter.write(newline);
				
			}
			else{		//saving because project is created
				//header project information
				//	System.out.println("saving because project is created");
				systemFileWriter.write("#PIDE 1.0- PIC Integrated Development Enviroment with ASM++");
				systemFileWriter.write(newline);
				systemFileWriter.write("#Project_Name= "+ fileName);
				systemFileWriter.write(newline);
				//!!find a way to get user of the system
				systemFileWriter.write("#Creator= ");
				systemFileWriter.write(newline);
				systemFileWriter.write("#Created@ " + new Date().toString());
				systemFileWriter.write(newline);
				systemFileWriter.write("#Modified@ " + new Date().toString());
				systemFileWriter.write(newline);
				systemFileWriter.write("#Description= " +projectDescription);
				systemFileWriter.write(newline);	
				systemFileWriter.write("#Files");
				systemFileWriter.write(newline);
				
				
				fileBuffer += "PIDE 1.0- PIC Integrated Development Enviroment with ASM++"+newline
							+"#Project_Name= "+ fileName+newline
							+"#Creator= " + newline 
							+"#Created@ " + new Date().toString()+newline
							+"#Modified@ " + new Date().toString()+newline;
			}
			

			for (int i = 0; i < fileList.size(); i++) {
				
				line = fileList.get(i);
				
					systemFileWriter.write(">FILE= "+line);
					systemFileWriter.write(newline);							
			}
			systemFileWriter.write("WORKSPACE_BEGIN");
			systemFileWriter.write(newline);
			
			systemFileWriter.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(0);
		}
	}

	public String getFilePath()
	{
		return filePath;
	}
}
