/**
 * 
 */
package projectManager;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;

/**
 * This class should be unique since there is only one 
 * system file (pide.sys). It holds information about system settings 
 * and program's latest snapshot.
 * 
 * Currently, system allows a maximum of 50 workspaces active at a time.
 * @author maho
 *
 */
public class SystemSettingsFile extends PIDEFile {
	
	//private static final int MAX_ACTIVE_WORKSPACES_ALLOWED = 50;
	/**Array of Strings that holds the names of the active workspaces*/
	public ArrayList<String> activeWorkspaces;
	public String activeProject = null;
	public boolean existsActiveProject = false;
	
	//public int noOfActiveWorkspaces;

	public SystemSettingsFile(int fileType, String fileName, String filePath) {
		super(fileType, fileName, filePath);
		// TODO Auto-generated constructor stub
		activeWorkspaces = new ArrayList<String>();//new String[MAX_ACTIVE_WORKSPACES_ALLOWED];
		/*for(int i=0;i<MAX_ACTIVE_WORKSPACES_ALLOWED;i++)
			activeWorkspaces[i] = new String();
			*/
		load();
	}

	/* (non-Javadoc)
	 * @see projectManager.PIDEFile#load()
	 */
	/**
	 * For system files, load operation consists of
	 * getting data from system file.
	 */
	@Override
	public void load() {
		// TODO Auto-generated method stub
		File systemFile  = new File(filePath + fileName + ".sys");
		BufferedReader inputStream = null;
		
		try {
			inputStream = new BufferedReader(new FileReader(systemFile));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			System.out.println("System file missing! The system file will be restored but information");
			System.out.println("is lost. You can recover workspace information after consulting PIDE");
			System.out.println("documents.");
			e.printStackTrace();
		}
		// Here only workspace information is extracted, most of the other initializations are 
		// skipped. They should be implemented later.
		String line = new String();
		fileBuffer = new String();
		try {
			//skip comments and other info
			do{
				line = inputStream.readLine();
				fileBuffer += line; //get very first few comments that will never change after the file is created
				fileBuffer += "\n";
			}
			while(line.compareTo("#Active Workspace")!=0);
			
				line = inputStream.readLine();
				if(line.compareTo("#Latest Workspaces") != 0)//there is an active project
				{
					//System.out.println(line);
					activeProject = new String(line);
					existsActiveProject = true;
					line = inputStream.readLine();//read line "#Active Workspace"
					//after finding the spot, get active workspace and all other workspaces
					
					for(;;)
					{
						line = inputStream.readLine();
						if(line.charAt(0) == '#')//another comment comes. End of Workspace list
							break;
						else 
						{
							activeWorkspaces.add(line);
						}
					}
				}
				
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	/* (non-Javadoc)
	 * @see projectManager.PIDEFile#save()
	 */
	/**
	 * This method simply saves all data into system settings file.
	 */
	@Override
	public void save() {
		// TODO Auto-generated method stub
		
		File systemFile  = new File(filePath + fileName + ".sys");
		FileWriter systemFileWriter = null;
		try {
			systemFileWriter = new FileWriter(systemFile);
		
		} catch (IOException e) {
			// TODO Auto-generated catch block
			System.out.println("System file missing! The system file will be restored but information");
			System.out.println("is lost. You can recover workspace information after consulting PIDE");
			System.out.println("documents.");
			e.printStackTrace();
		}
		try {
			systemFileWriter.write(fileBuffer);//initial comments
			
			//active workspace
			if(existsActiveProject)
			{
				systemFileWriter.write(activeProject);
				systemFileWriter.write("\n");
			}
			//latest workspaces
			systemFileWriter.write("#Latest Workspaces");
			systemFileWriter.write("\n");
			for(int i=0; i<activeWorkspaces.size() ; i++ )
			{
				systemFileWriter.write(activeWorkspaces.get(i));
				systemFileWriter.write("\n");			
			}
			
			//Latest System Settings 			
			systemFileWriter.write("#Latest System Settings");
			systemFileWriter.write("\n");
			systemFileWriter.write("#set on " + new Date().toString());
			systemFileWriter.write("\n");
		
			//Recent files
			systemFileWriter.write("#Recent files");
			systemFileWriter.write("\n");
			systemFileWriter.write("recent_documents:");
			systemFileWriter.write("\n");
			systemFileWriter.write("recent_projects:");
			systemFileWriter.write("\n");
	
			//Editor Settings
			systemFileWriter.write("#Editor Settings");
			systemFileWriter.write("\n");
			
			systemFileWriter.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

}
